@@ -1,10 +1,13 @@
 /*
- * Copyright (c) 2018 Yubico AB. All rights reserved.
+ * Copyright (c) 2018-2021 Yubico AB. All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the LICENSE file.
  */
 
-#include <openssl/ec.h>
+#include <fido.h>
+#include <fido/es256.h>
+#include <fido/rs256.h>
+#include <fido/eddsa.h>
 
 #include <stdbool.h>
 #include <stdio.h>
@@ -14,14 +17,10 @@
 #include <unistd.h>
 #endif
 
-#include "fido.h"
-#include "fido/es256.h"
-#include "fido/rs256.h"
-#include "fido/eddsa.h"
-#include "extern.h"
 #include "../openbsd-compat/openbsd-compat.h"
+#include "extern.h"
 
-static const unsigned char cdh[32] = {
+static const unsigned char cd[32] = {
 	0xec, 0x8d, 0x8f, 0x78, 0x42, 0x4a, 0x2b, 0xb7,
 	0x82, 0x34, 0xaa, 0xca, 0x07, 0xa1, 0xf6, 0x56,
 	0x42, 0x1c, 0xb6, 0xf6, 0xb3, 0x00, 0x86, 0x52,
@@ -32,8 +31,8 @@ static void
 usage(void)
 {
 	fprintf(stderr, "usage: assert [-t ecdsa|rsa|eddsa] [-a cred_id] "
-	    "[-h hmac_secret] [-s hmac_salt] [-P pin] [-T seconds] [-puv] "
-	    "<pubkey> <device>\n");
+	    "[-h hmac_secret] [-s hmac_salt] [-P pin] [-T seconds] "
+	    "[-b blobkey] [-puv] <pubkey> <device>\n");
 	exit(EXIT_FAILURE);
 }
 
@@ -107,10 +106,9 @@ verify_assert(int type, const unsigned char *authdata_ptr, size_t authdata_len,
 		errx(1, "fido_assert_new");
 
 	/* client data hash */
-	r = fido_assert_set_clientdata_hash(assert, cdh, sizeof(cdh));
+	r = fido_assert_set_clientdata(assert, cd, sizeof(cd));
 	if (r != FIDO_OK)
-		errx(1, "fido_assert_set_clientdata_hash: %s (0x%x)",
-		    fido_strerr(r), r);
+		errx(1, "fido_assert_set_clientdata: %s (0x%x)", fido_strerr(r), r);
 
 	/* relying party */
 	r = fido_assert_set_rp(assert, "localhost");
@@ -164,9 +162,10 @@ main(int argc, char **argv)
 	fido_dev_t	*dev = NULL;
 	fido_assert_t	*assert = NULL;
 	const char	*pin = NULL;
+	const char	*blobkey_out = NULL;
 	const char	*hmac_out = NULL;
 	unsigned char	*body = NULL;
-	long long	 seconds = 0;
+	long long	 ms = 0;
 	size_t		 len;
 	int		 type = COSE_ES256;
 	int		 ext = 0;
@@ -176,22 +175,18 @@ main(int argc, char **argv)
 	if ((assert = fido_assert_new()) == NULL)
 		errx(1, "fido_assert_new");
 
-	while ((ch = getopt(argc, argv, "P:T:a:h:ps:t:uv")) != -1) {
+	while ((ch = getopt(argc, argv, "P:T:a:b:h:ps:t:uv")) != -1) {
 		switch (ch) {
 		case 'P':
 			pin = optarg;
 			break;
 		case 'T':
-#ifndef SIGNAL_EXAMPLE
-			(void)seconds;
-			errx(1, "-T not supported");
-#else
-			if (base10(optarg, &seconds) < 0)
+			if (base10(optarg, &ms) < 0)
 				errx(1, "base10: %s", optarg);
-			if (seconds <= 0 || seconds > 30)
+			if (ms <= 0 || ms > 30)
 				errx(1, "-T: %s must be in (0,30]", optarg);
+			ms *= 1000; /* seconds to milliseconds */
 			break;
-#endif
 		case 'a':
 			if (read_blob(optarg, &body, &len) < 0)
 				errx(1, "read_blob: %s", optarg);
@@ -202,6 +197,10 @@ main(int argc, char **argv)
 			free(body);
 			body = NULL;
 			break;
+		case 'b':
+			ext |= FIDO_EXT_LARGEBLOB_KEY;
+			blobkey_out = optarg;
+			break;
 		case 'h':
 			hmac_out = optarg;
 			break;
@@ -209,7 +208,7 @@ main(int argc, char **argv)
 			up = true;
 			break;
 		case 's':
-			ext = FIDO_EXT_HMAC_SECRET;
+			ext |= FIDO_EXT_HMAC_SECRET;
 			if (read_blob(optarg, &body, &len) < 0)
 				errx(1, "read_blob: %s", optarg);
 			if ((r = fido_assert_set_hmac_salt(assert, body,
@@ -258,10 +257,9 @@ main(int argc, char **argv)
 		fido_dev_force_u2f(dev);
 
 	/* client data hash */
-	r = fido_assert_set_clientdata_hash(assert, cdh, sizeof(cdh));
+	r = fido_assert_set_clientdata(assert, cd, sizeof(cd));
 	if (r != FIDO_OK)
-		errx(1, "fido_assert_set_clientdata_hash: %s (0x%x)",
-		    fido_strerr(r), r);
+		errx(1, "fido_assert_set_clientdata: %s (0x%x)", fido_strerr(r), r);
 
 	/* relying party */
 	r = fido_assert_set_rp(assert, "localhost");
@@ -282,20 +280,12 @@ main(int argc, char **argv)
 	if (uv && (r = fido_assert_set_uv(assert, FIDO_OPT_TRUE)) != FIDO_OK)
 		errx(1, "fido_assert_set_uv: %s (0x%x)", fido_strerr(r), r);
 
-#ifdef SIGNAL_EXAMPLE
-	prepare_signal_handler(SIGINT);
-	if (seconds) {
-		prepare_signal_handler(SIGALRM);
-		alarm((unsigned)seconds);
-	}
-#endif
+	/* timeout */
+	if (ms != 0 && (r = fido_dev_set_timeout(dev, (int)ms)) != FIDO_OK)
+		errx(1, "fido_dev_set_timeout: %s (0x%x)", fido_strerr(r), r);
 
-	r = fido_dev_get_assert(dev, assert, pin);
-	if (r != FIDO_OK) {
-#ifdef SIGNAL_EXAMPLE
-		if (got_signal)
-			fido_dev_cancel(dev);
-#endif
+	if ((r = fido_dev_get_assert(dev, assert, pin)) != FIDO_OK) {
+		fido_dev_cancel(dev);
 		errx(1, "fido_dev_get_assert: %s (0x%x)", fido_strerr(r), r);
 	}
 
@@ -324,6 +314,14 @@ main(int argc, char **argv)
 			errx(1, "write_blob");
 	}
 
+	if (blobkey_out != NULL) {
+		/* extract the hmac secret */
+		if (write_blob(blobkey_out,
+		    fido_assert_largeblob_key_ptr(assert, 0),
+		    fido_assert_largeblob_key_len(assert, 0)) < 0)
+			errx(1, "write_blob");
+	}
+
 	fido_assert_free(&assert);
 
 	exit(0);
