@@ -11,10 +11,6 @@
 #include <openssl/evp.h>
 #include <openssl/rsa.h>
 
-#ifdef HAVE_SIGNAL_H
-#include <signal.h>
-#endif
-
 /* util.c */
 EC_KEY *read_ec_pubkey(const char *);
 RSA *read_rsa_pubkey(const  char *);
@@ -25,9 +21,5 @@ int write_blob(const char *, const unsigned char *, size_t);
 int write_ec_pubkey(const char *, const void *, size_t);
 int write_rsa_pubkey(const char *, const void *, size_t);
 int write_eddsa_pubkey(const char *, const void *, size_t);
-#ifdef SIGNAL_EXAMPLE
-void prepare_signal_handler(int);
-extern volatile sig_atomic_t got_signal;
-#endif
 
 #endif /* _EXTERN_H_ */
