@@ -4,11 +4,10 @@
  * license that can be found in the LICENSE file.
  */
 
-#include <stdbool.h>
+#include <fido.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "fido.h"
 #include "../openbsd-compat/openbsd-compat.h"
 
 int
