@@ -6,17 +6,14 @@
 
 #include <assert.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdio.h>
 
 #include "mutator_aux.h"
 #include "wiredata_fido2.h"
 #include "dummy.h"
 
-#include "fido.h"
-#include "fido/bio.h"
-
 #include "../openbsd-compat/openbsd-compat.h"
 
 /* Parameter set defining a FIDO2 credential management operation. */
@@ -223,21 +220,10 @@ static fido_dev_t *
 prepare_dev(void)
 {
 	fido_dev_t *dev;
-	fido_dev_io_t io;
 	bool x;
 
-	memset(&io, 0, sizeof(io));
-
-	io.open = dev_open;
-	io.close = dev_close;
-	io.read = dev_read;
-	io.write = dev_write;
-
-	if ((dev = fido_dev_new()) == NULL || fido_dev_set_io_functions(dev,
-	    &io) != FIDO_OK || fido_dev_open(dev, "nodev") != FIDO_OK) {
-		fido_dev_free(&dev);
+	if ((dev = open_dev(0)) == NULL)
 		return NULL;
-	}
 
 	x = fido_dev_is_fido2(dev);
 	consume(&x, sizeof(x));
@@ -245,6 +231,10 @@ prepare_dev(void)
 	consume(&x, sizeof(x));
 	x = fido_dev_has_pin(dev);
 	consume(&x, sizeof(x));
+	x = fido_dev_supports_uv(dev);
+	consume(&x, sizeof(x));
+	x = fido_dev_has_uv(dev);
+	consume(&x, sizeof(x));
 
 	return dev;
 }
@@ -282,7 +272,7 @@ done:
 static void
 consume_template(const fido_bio_template_t *t)
 {
-	consume(fido_bio_template_name(t), xstrlen(fido_bio_template_name(t)));
+	consume_str(fido_bio_template_name(t));
 	consume(fido_bio_template_id_ptr(t), fido_bio_template_id_len(t));
 }
 
@@ -313,7 +303,7 @@ enroll(const struct param *p)
 	    (e = fido_bio_enroll_new()) == NULL)
 		goto done;
 
-	fido_bio_dev_enroll_begin(dev, t, e, p->seed, p->pin);
+	fido_bio_dev_enroll_begin(dev, t, e, (uint32_t)p->seed, p->pin);
 
 	consume_template(t);
 	consume_enroll(e);
@@ -418,6 +408,7 @@ void
 test(const struct param *p)
 {
 	prng_init((unsigned int)p->seed);
+	fuzz_clock_reset();
 	fido_init(FIDO_DEBUG);
 	fido_set_log_handler(consume_str);
 
