@@ -1,20 +1,19 @@
 /*
- * Copyright (c) 2019 Yubico AB. All rights reserved.
+ * Copyright (c) 2019-2021 Yubico AB. All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the LICENSE file.
  */
 
 #include <assert.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdio.h>
 
 #include "mutator_aux.h"
 #include "wiredata_fido2.h"
 #include "wiredata_u2f.h"
 #include "dummy.h"
-#include "fido.h"
 
 #include "../openbsd-compat/openbsd-compat.h"
 
@@ -35,7 +34,7 @@ struct param {
 	uint8_t excl_count;
 	uint8_t rk;
 	uint8_t type;
-	uint8_t u2f;
+	uint8_t opt;
 	uint8_t uv;
 };
 
@@ -87,7 +86,7 @@ unpack(const uint8_t *ptr, size_t len)
 
 	if (unpack_byte(v[0], &p->rk) < 0 ||
 	    unpack_byte(v[1], &p->type) < 0 ||
-	    unpack_byte(v[2], &p->u2f) < 0 ||
+	    unpack_byte(v[2], &p->opt) < 0 ||
 	    unpack_byte(v[3], &p->uv) < 0 ||
 	    unpack_byte(v[4], &p->excl_count) < 0 ||
 	    unpack_int(v[5], &p->ext) < 0 ||
@@ -129,7 +128,7 @@ pack(uint8_t *ptr, size_t len, const struct param *p)
 	if ((array = cbor_new_definite_array(17)) == NULL ||
 	    (argv[0] = pack_byte(p->rk)) == NULL ||
 	    (argv[1] = pack_byte(p->type)) == NULL ||
-	    (argv[2] = pack_byte(p->u2f)) == NULL ||
+	    (argv[2] = pack_byte(p->opt)) == NULL ||
 	    (argv[3] = pack_byte(p->uv)) == NULL ||
 	    (argv[4] = pack_byte(p->excl_count)) == NULL ||
 	    (argv[5] = pack_int(p->ext)) == NULL ||
@@ -211,29 +210,17 @@ pack_dummy(uint8_t *ptr, size_t len)
 }
 
 static void
-make_cred(fido_cred_t *cred, uint8_t u2f, int type, const struct blob *cdh,
+make_cred(fido_cred_t *cred, uint8_t opt, int type, const struct blob *cdh,
     const char *rp_id, const char *rp_name, const struct blob *user_id,
     const char *user_name, const char *user_nick, const char *user_icon,
     int ext, uint8_t rk, uint8_t uv, const char *pin, uint8_t excl_count,
     const struct blob *excl_cred)
 {
 	fido_dev_t *dev;
-	fido_dev_io_t io;
 
-	memset(&io, 0, sizeof(io));
-
-	io.open = dev_open;
-	io.close = dev_close;
-	io.read = dev_read;
-	io.write = dev_write;
-
-	if ((dev = fido_dev_new()) == NULL || fido_dev_set_io_functions(dev,
-	    &io) != FIDO_OK || fido_dev_open(dev, "nodev") != FIDO_OK) {
-		fido_dev_free(&dev);
+	if ((dev = open_dev(opt & 2)) == NULL)
 		return;
-	}
-
-	if (u2f & 1)
+	if (opt & 1)
 		fido_dev_force_u2f(dev);
 
 	for (uint8_t i = 0; i < excl_count; i++)
@@ -244,7 +231,15 @@ make_cred(fido_cred_t *cred, uint8_t u2f, int type, const struct blob *cdh,
 	fido_cred_set_rp(cred, rp_id, rp_name);
 	fido_cred_set_user(cred, user_id->body, user_id->len, user_name,
 	    user_nick, user_icon);
-	fido_cred_set_extensions(cred, ext);
+
+	if (ext & FIDO_EXT_HMAC_SECRET)
+		fido_cred_set_extensions(cred, FIDO_EXT_HMAC_SECRET);
+	if (ext & FIDO_EXT_CRED_BLOB)
+		fido_cred_set_blob(cred, user_id->body, user_id->len);
+	if (ext & FIDO_EXT_LARGEBLOB_KEY)
+		fido_cred_set_extensions(cred, FIDO_EXT_LARGEBLOB_KEY);
+	if (ext & FIDO_EXT_MINPINLEN)
+		fido_cred_set_pin_minlen(cred, strlen(pin));
 
 	if (rk & 1)
 		fido_cred_set_rk(cred, FIDO_OPT_TRUE);
@@ -263,7 +258,7 @@ make_cred(fido_cred_t *cred, uint8_t u2f, int type, const struct blob *cdh,
 	if (strlen(pin) == 0)
 		pin = NULL;
 
-	fido_dev_make_cred(dev, cred, u2f & 1 ? NULL : pin);
+	fido_dev_make_cred(dev, cred, (opt & 1) ? NULL : pin);
 
 	fido_dev_cancel(dev);
 	fido_dev_close(dev);
@@ -276,11 +271,13 @@ verify_cred(int type, const unsigned char *cdh_ptr, size_t cdh_len,
     size_t authdata_len, const unsigned char *authdata_raw_ptr,
     size_t authdata_raw_len, int ext, uint8_t rk, uint8_t uv,
     const unsigned char *x5c_ptr, size_t x5c_len, const unsigned char *sig_ptr,
-    size_t sig_len, const char *fmt, int prot)
+    size_t sig_len, const unsigned char *attstmt_ptr, size_t attstmt_len,
+    const char *fmt, int prot, size_t minpinlen)
 {
 	fido_cred_t *cred;
 	uint8_t flags;
 	uint32_t sigcount;
+	int r;
 
 	if ((cred = fido_cred_new()) == NULL)
 		return;
@@ -290,13 +287,19 @@ verify_cred(int type, const unsigned char *cdh_ptr, size_t cdh_len,
 	fido_cred_set_rp(cred, rp_id, rp_name);
 	consume(authdata_ptr, authdata_len);
 	consume(authdata_raw_ptr, authdata_raw_len);
+	consume(x5c_ptr, x5c_len);
+	consume(sig_ptr, sig_len);
+	consume(attstmt_ptr, attstmt_len);
 	if (fido_cred_set_authdata(cred, authdata_ptr, authdata_len) != FIDO_OK)
 		fido_cred_set_authdata_raw(cred, authdata_raw_ptr,
 		    authdata_raw_len);
 	fido_cred_set_extensions(cred, ext);
-	fido_cred_set_x509(cred, x5c_ptr, x5c_len);
-	fido_cred_set_sig(cred, sig_ptr, sig_len);
+	if (fido_cred_set_attstmt(cred, attstmt_ptr, attstmt_len) != FIDO_OK) {
+		fido_cred_set_x509(cred, x5c_ptr, x5c_len);
+		fido_cred_set_sig(cred, sig_ptr, sig_len);
+	}
 	fido_cred_set_prot(cred, prot);
+	fido_cred_set_pin_minlen(cred, minpinlen);
 
 	if (rk & 1)
 		fido_cred_set_rk(cred, FIDO_OPT_TRUE);
@@ -307,20 +310,28 @@ verify_cred(int type, const unsigned char *cdh_ptr, size_t cdh_len,
 
 	/* repeat memory operations to trigger reallocation paths */
 	if (fido_cred_set_authdata(cred, authdata_ptr, authdata_len) != FIDO_OK)
-		fido_cred_set_authdata_raw(cred, authdata_ptr, authdata_len);
+		fido_cred_set_authdata_raw(cred, authdata_raw_ptr,
+		    authdata_raw_len);
+	if (fido_cred_set_attstmt(cred, attstmt_ptr, attstmt_len) != FIDO_OK) {
+		fido_cred_set_x509(cred, x5c_ptr, x5c_len);
+		fido_cred_set_sig(cred, sig_ptr, sig_len);
+	}
 	fido_cred_set_x509(cred, x5c_ptr, x5c_len);
 	fido_cred_set_sig(cred, sig_ptr, sig_len);
 
-	assert(fido_cred_verify(cred) != FIDO_OK);
-	assert(fido_cred_verify_self(cred) != FIDO_OK);
+	r = fido_cred_verify(cred);
+	consume(&r, sizeof(r));
+	r = fido_cred_verify_self(cred);
+	consume(&r, sizeof(r));
 
 	consume(fido_cred_pubkey_ptr(cred), fido_cred_pubkey_len(cred));
 	consume(fido_cred_id_ptr(cred), fido_cred_id_len(cred));
 	consume(fido_cred_aaguid_ptr(cred), fido_cred_aaguid_len(cred));
 	consume(fido_cred_user_id_ptr(cred), fido_cred_user_id_len(cred));
-	consume(fido_cred_user_name(cred), xstrlen(fido_cred_user_name(cred)));
-	consume(fido_cred_display_name(cred),
-	    xstrlen(fido_cred_display_name(cred)));
+	consume_str(fido_cred_user_name(cred));
+	consume_str(fido_cred_display_name(cred));
+	consume(fido_cred_largeblob_key_ptr(cred),
+	    fido_cred_largeblob_key_len(cred));
 
 	flags = fido_cred_flags(cred);
 	consume(&flags, sizeof(flags));
@@ -328,6 +339,8 @@ verify_cred(int type, const unsigned char *cdh_ptr, size_t cdh_len,
 	consume(&sigcount, sizeof(sigcount));
 	type = fido_cred_type(cred);
 	consume(&type, sizeof(type));
+	minpinlen = fido_cred_pin_minlen(cred);
+	consume(&minpinlen, sizeof(minpinlen));
 
 	fido_cred_free(&cred);
 }
@@ -355,7 +368,7 @@ test_cred(const struct param *p)
 
 	set_wire_data(p->wire_data.body, p->wire_data.len);
 
-	make_cred(cred, p->u2f, cose_alg, &p->cdh, p->rp_id, p->rp_name,
+	make_cred(cred, p->opt, cose_alg, &p->cdh, p->rp_id, p->rp_name,
 	    &p->user_id, p->user_name, p->user_nick, p->user_icon, p->ext,
 	    p->rk, p->uv, p->pin, p->excl_count, &p->excl_cred);
 
@@ -367,7 +380,9 @@ test_cred(const struct param *p)
 	    fido_cred_authdata_raw_len(cred), p->ext, p->rk, p->uv,
 	    fido_cred_x5c_ptr(cred), fido_cred_x5c_len(cred),
 	    fido_cred_sig_ptr(cred), fido_cred_sig_len(cred),
-	    fido_cred_fmt(cred), fido_cred_prot(cred));
+	    fido_cred_attstmt_ptr(cred), fido_cred_attstmt_len(cred),
+	    fido_cred_fmt(cred), fido_cred_prot(cred),
+	    fido_cred_pin_minlen(cred));
 
 	fido_cred_free(&cred);
 }
@@ -376,26 +391,14 @@ static void
 test_touch(const struct param *p)
 {
 	fido_dev_t *dev;
-	fido_dev_io_t io;
 	int r;
 	int touched;
 
-	memset(&io, 0, sizeof(io));
-
-	io.open = dev_open;
-	io.close = dev_close;
-	io.read = dev_read;
-	io.write = dev_write;
-
 	set_wire_data(p->wire_data.body, p->wire_data.len);
 
-	if ((dev = fido_dev_new()) == NULL || fido_dev_set_io_functions(dev,
-	    &io) != FIDO_OK || fido_dev_open(dev, "nodev") != FIDO_OK) {
-		fido_dev_free(&dev);
+	if ((dev = open_dev(p->opt & 2)) == NULL)
 		return;
-	}
-
-	if (p->u2f & 1)
+	if (p->opt & 1)
 		fido_dev_force_u2f(dev);
 
 	r = fido_dev_get_touch_begin(dev);
@@ -409,15 +412,31 @@ test_touch(const struct param *p)
 	fido_dev_free(&dev);
 }
 
+static void
+test_misc(const struct param *p)
+{
+	fido_cred_t *cred = NULL;
+
+	if ((cred = fido_cred_new()) == NULL)
+		return;
+
+	/* reuse user id as credential id */
+	fido_cred_set_id(cred, p->user_id.body, p->user_id.len);
+	consume(fido_cred_id_ptr(cred), fido_cred_id_len(cred));
+	fido_cred_free(&cred);
+}
+
 void
 test(const struct param *p)
 {
 	prng_init((unsigned int)p->seed);
+	fuzz_clock_reset();
 	fido_init(FIDO_DEBUG);
 	fido_set_log_handler(consume_str);
 
 	test_cred(p);
 	test_touch(p);
+	test_misc(p);
 }
 
 void
@@ -429,7 +448,7 @@ mutate(struct param *p, unsigned int seed, unsigned int flags) NO_MSAN
 	if (flags & MUTATE_PARAM) {
 		mutate_byte(&p->rk);
 		mutate_byte(&p->type);
-		mutate_byte(&p->u2f);
+		mutate_byte(&p->opt);
 		mutate_byte(&p->uv);
 		mutate_byte(&p->excl_count);
 		mutate_int(&p->ext);
@@ -445,7 +464,7 @@ mutate(struct param *p, unsigned int seed, unsigned int flags) NO_MSAN
 	}
 
 	if (flags & MUTATE_WIREDATA) {
-		if (p->u2f & 1) {
+		if (p->opt & 1) {
 			p->wire_data.len = sizeof(dummy_wire_data_u2f);
 			memcpy(&p->wire_data.body, &dummy_wire_data_u2f,
 			    p->wire_data.len);
