@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2019 Yubico AB. All rights reserved.
+ * Copyright (c) 2019-2021 Yubico AB. All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the LICENSE file.
  */
 
+#include <sys/types.h>
+#include <sys/socket.h>
+
 #include <openssl/bn.h>
 #include <openssl/evp.h>
 #include <openssl/sha.h>
 
 #include <cbor.h>
-#include <fido.h>
-
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -52,6 +53,14 @@ WRAP(void *,
 	1
 )
 
+WRAP(void *,
+	realloc,
+	(void *ptr, size_t size),
+	NULL,
+	(ptr, size),
+	1
+)
+
 WRAP(char *,
 	strdup,
 	(const char *s),
@@ -60,6 +69,23 @@ WRAP(char *,
 	1
 )
 
+WRAP(int,
+	EVP_Cipher,
+	(EVP_CIPHER_CTX *ctx, unsigned char *out, const unsigned char *in,
+	    unsigned int inl),
+	-1,
+	(ctx, out, in, inl),
+	1
+)
+
+WRAP(int,
+	EVP_CIPHER_CTX_ctrl,
+	(EVP_CIPHER_CTX *ctx, int type, int arg, void *ptr),
+	0,
+	(ctx, type, arg, ptr),
+	1
+)
+
 WRAP(EVP_CIPHER_CTX *,
 	EVP_CIPHER_CTX_new,
 	(void),
@@ -68,70 +94,12 @@ WRAP(EVP_CIPHER_CTX *,
 	1
 )
 
-WRAP(int, EVP_EncryptInit_ex,
-	(EVP_CIPHER_CTX *ctx, const EVP_CIPHER *type, ENGINE *impl,
-	    const unsigned char *key, const unsigned char *iv),
-	0,
-	(ctx, type, impl, key, iv),
-	1
-)
-
 WRAP(int,
-	EVP_CIPHER_CTX_set_padding,
-	(EVP_CIPHER_CTX *x, int padding),
+	EVP_CipherInit,
+	(EVP_CIPHER_CTX *ctx, const EVP_CIPHER *cipher,
+	    const unsigned char *key, const unsigned char *iv, int enc),
 	0,
-	(x, padding),
-	1
-)
-
-WRAP(int,
-	EVP_EncryptUpdate,
-	(EVP_CIPHER_CTX *ctx, unsigned char *out, int *outl,
-	    const unsigned char *in, int inl),
-	0,
-	(ctx, out, outl, in, inl),
-	1
-)
-
-WRAP(int,
-	EVP_DecryptInit_ex,
-	(EVP_CIPHER_CTX *ctx, const EVP_CIPHER *type, ENGINE *impl,
-	    const unsigned char *key, const unsigned char *iv),
-	0,
-	(ctx, type, impl, key, iv),
-	1
-)
-
-WRAP(int,
-	EVP_DecryptUpdate,
-	(EVP_CIPHER_CTX *ctx, unsigned char *out, int *outl,
-	    const unsigned char *in, int inl),
-	0,
-	(ctx, out, outl, in, inl),
-	1
-)
-
-WRAP(int,
-	SHA256_Init,
-	(SHA256_CTX *c),
-	0,
-	(c),
-	1
-)
-
-WRAP(int,
-	SHA256_Update,
-	(SHA256_CTX *c, const void *data, size_t len),
-	0,
-	(c, data, len),
-	1
-)
-
-WRAP(int,
-	SHA256_Final,
-	(unsigned char *md, SHA256_CTX *c),
-	0,
-	(md, c),
+	(ctx, cipher, key, iv, enc),
 	1
 )
 
@@ -176,6 +144,30 @@ WRAP(int,
 	1
 )
 
+WRAP(int,
+	EVP_DigestInit_ex,
+	(EVP_MD_CTX *ctx, const EVP_MD *type, ENGINE *impl),
+	0,
+	(ctx, type, impl),
+	1
+)
+
+WRAP(int,
+	EVP_DigestUpdate,
+	(EVP_MD_CTX *ctx, const void *data, size_t count),
+	0,
+	(ctx, data, count),
+	1
+)
+
+WRAP(int,
+	EVP_DigestFinal_ex,
+	(EVP_MD_CTX *ctx, unsigned char *md, unsigned int *isize),
+	0,
+	(ctx, md, isize),
+	1
+)
+
 WRAP(BIGNUM *,
 	BN_bin2bn,
 	(const unsigned char *s, int len, BIGNUM *ret),
@@ -216,6 +208,14 @@ WRAP(BIGNUM *,
 	1
 )
 
+WRAP(RSA *,
+	RSA_new,
+	(void),
+	NULL,
+	(),
+	1
+)
+
 WRAP(int,
 	RSA_set0_key,
 	(RSA *r, BIGNUM *n, BIGNUM *e, BIGNUM *d),
@@ -224,6 +224,14 @@ WRAP(int,
 	1
 )
 
+WRAP(int,
+	RSA_pkey_ctx_ctrl,
+	(EVP_PKEY_CTX *ctx, int optype, int cmd, int p1, void *p2),
+	-1,
+	(ctx, optype, cmd, p1, p2),
+	1
+)
+
 WRAP(EC_KEY *,
 	EC_KEY_new_by_curve_name,
 	(int nid),
@@ -336,6 +344,14 @@ WRAP(EVP_PKEY_CTX *,
 	1
 )
 
+WRAP(int,
+	EVP_PKEY_derive,
+	(EVP_PKEY_CTX *ctx, unsigned char *key, size_t *pkeylen),
+	0,
+	(ctx, key, pkeylen),
+	1
+)
+
 WRAP(int,
 	EVP_PKEY_derive_init,
 	(EVP_PKEY_CTX *ctx),
@@ -352,6 +368,30 @@ WRAP(int,
 	1
 )
 
+WRAP(int,
+	EVP_PKEY_verify_init,
+	(EVP_PKEY_CTX *ctx),
+	0,
+	(ctx),
+	1
+)
+
+WRAP(int,
+	EVP_PKEY_CTX_ctrl,
+	(EVP_PKEY_CTX *ctx, int keytype, int optype, int cmd, int p1, void *p2),
+	-1,
+	(ctx, keytype, optype, cmd, p1, p2),
+	1
+)
+
+WRAP(const EVP_MD *,
+	EVP_sha1,
+	(void),
+	NULL,
+	(),
+	1
+)
+
 WRAP(const EVP_MD *,
 	EVP_sha256,
 	(void),
@@ -360,6 +400,22 @@ WRAP(const EVP_MD *,
 	1
 )
 
+WRAP(const EVP_CIPHER *,
+	EVP_aes_256_cbc,
+	(void),
+	NULL,
+	(),
+	1
+)
+
+WRAP(const EVP_CIPHER *,
+	EVP_aes_256_gcm,
+	(void),
+	NULL,
+	(),
+	1
+)
+
 WRAP(unsigned char *,
 	HMAC,
 	(const EVP_MD *evp_md, const void *key, int key_len,
@@ -403,6 +459,14 @@ WRAP(int,
 	1
 )
 
+WRAP(unsigned char *,
+	SHA1,
+	(const unsigned char *d, size_t n, unsigned char *md),
+	NULL,
+	(d, n, md),
+	1
+)
+
 WRAP(unsigned char *,
 	SHA256,
 	(const unsigned char *d, size_t n, unsigned char *md),
@@ -467,6 +531,14 @@ WRAP(cbor_item_t *,
 	1
 )
 
+WRAP(cbor_item_t *,
+	cbor_build_uint16,
+	(uint16_t value),
+	NULL,
+	(value),
+	1
+)
+
 WRAP(cbor_item_t *,
 	cbor_build_uint32,
 	(uint32_t value),
@@ -475,6 +547,14 @@ WRAP(cbor_item_t *,
 	1
 )
 
+WRAP(cbor_item_t *,
+	cbor_build_uint64,
+	(uint64_t value),
+	NULL,
+	(value),
+	1
+)
+
 WRAP(struct cbor_pair *,
 	cbor_map_handle,
 	(const cbor_item_t *item),
@@ -523,6 +603,14 @@ WRAP(cbor_item_t *,
 	1
 )
 
+WRAP(cbor_item_t *,
+	cbor_new_definite_bytestring,
+	(void),
+	NULL,
+	(),
+	1
+)
+
 WRAP(size_t,
 	cbor_serialize_alloc,
 	(const cbor_item_t *item, cbor_mutable_data *buffer,
@@ -534,16 +622,16 @@ WRAP(size_t,
 
 WRAP(int,
 	fido_tx,
-	(fido_dev_t *d, uint8_t cmd, const void *buf, size_t count),
+	(fido_dev_t *d, uint8_t cmd, const void *buf, size_t count, int *ms),
 	-1,
-	(d, cmd, buf, count),
+	(d, cmd, buf, count, ms),
 	1
 )
 
 WRAP(int,
-	usleep,
-	(unsigned int usec),
+	bind,
+	(int sockfd, const struct sockaddr *addr, socklen_t addrlen),
 	-1,
-	(usec),
+	(sockfd, addr, addrlen),
 	1
 )
