@@ -4,8 +4,6 @@
  * license that can be found in the LICENSE file.
  */
 
-#include <string.h>
-
 #include "fido.h"
 #include "fido/bio.h"
 #include "fido/es256.h"
@@ -59,14 +57,15 @@ fail:
 }
 
 static int
-bio_tx(fido_dev_t *dev, uint8_t cmd, cbor_item_t **sub_argv, size_t sub_argc,
-    const char *pin, const fido_blob_t *token)
+bio_tx(fido_dev_t *dev, uint8_t subcmd, cbor_item_t **sub_argv, size_t sub_argc,
+    const char *pin, const fido_blob_t *token, int *ms)
 {
 	cbor_item_t	*argv[5];
 	es256_pk_t	*pk = NULL;
 	fido_blob_t	*ecdh = NULL;
 	fido_blob_t	 f;
 	fido_blob_t	 hmac;
+	const uint8_t	 cmd = CTAP_CBOR_BIO_ENROLL_PRE;
 	int		 r = FIDO_ERR_INTERNAL;
 
 	memset(&f, 0, sizeof(f));
@@ -75,14 +74,14 @@ bio_tx(fido_dev_t *dev, uint8_t cmd, cbor_item_t **sub_argv, size_t sub_argc,
 
 	/* modality, subCommand */
 	if ((argv[0] = cbor_build_uint8(1)) == NULL ||
-	    (argv[1] = cbor_build_uint8(cmd)) == NULL) {
+	    (argv[1] = cbor_build_uint8(subcmd)) == NULL) {
 		fido_log_debug("%s: cbor encode", __func__);
 		goto fail;
 	}
 
 	/* subParams */
 	if (pin || token) {
-		if (bio_prepare_hmac(cmd, sub_argv, sub_argc, &argv[2],
+		if (bio_prepare_hmac(subcmd, sub_argv, sub_argc, &argv[2],
 		    &hmac) < 0) {
 			fido_log_debug("%s: bio_prepare_hmac", __func__);
 			goto fail;
@@ -91,26 +90,26 @@ bio_tx(fido_dev_t *dev, uint8_t cmd, cbor_item_t **sub_argv, size_t sub_argc,
 
 	/* pinProtocol, pinAuth */
 	if (pin) {
-		if ((r = fido_do_ecdh(dev, &pk, &ecdh)) != FIDO_OK) {
+		if ((r = fido_do_ecdh(dev, &pk, &ecdh, ms)) != FIDO_OK) {
 			fido_log_debug("%s: fido_do_ecdh", __func__);
 			goto fail;
 		}
-		if ((r = cbor_add_pin_params(dev, &hmac, pk, ecdh, pin,
-		    &argv[4], &argv[3])) != FIDO_OK) {
-			fido_log_debug("%s: cbor_add_pin_params", __func__);
+		if ((r = cbor_add_uv_params(dev, cmd, &hmac, pk, ecdh, pin,
+		    NULL, &argv[4], &argv[3], ms)) != FIDO_OK) {
+			fido_log_debug("%s: cbor_add_uv_params", __func__);
 			goto fail;
 		}
 	} else if (token) {
-		if ((argv[3] = cbor_encode_pin_opt()) == NULL ||
-		    (argv[4] = cbor_encode_pin_auth(token, &hmac)) == NULL) {
+		if ((argv[3] = cbor_encode_pin_opt(dev)) == NULL ||
+		    (argv[4] = cbor_encode_pin_auth(dev, token, &hmac)) == NULL) {
 			fido_log_debug("%s: encode pin", __func__);
 			goto fail;
 		}
 	}
 
 	/* framing and transmission */
-	if (cbor_build_frame(CTAP_CBOR_BIO_ENROLL_PRE, argv, 5, &f) < 0 ||
-	    fido_tx(dev, CTAP_FRAME_INIT | CTAP_CMD_CBOR, f.ptr, f.len) < 0) {
+	if (cbor_build_frame(cmd, argv, nitems(argv), &f) < 0 ||
+	    fido_tx(dev, CTAP_CMD_CBOR, f.ptr, f.len, ms) < 0) {
 		fido_log_debug("%s: fido_tx", __func__);
 		r = FIDO_ERR_TX;
 		goto fail;
@@ -131,9 +130,8 @@ static void
 bio_reset_template(fido_bio_template_t *t)
 {
 	free(t->name);
-	free(t->id.ptr);
 	t->name = NULL;
-	memset(&t->id, 0, sizeof(t->id));
+	fido_blob_reset(&t->id);
 }
 
 static void
@@ -233,16 +231,16 @@ bio_parse_template_array(const cbor_item_t *key, const cbor_item_t *val,
 }
 
 static int
-bio_rx_template_array(fido_dev_t *dev, fido_bio_template_array_t *ta, int ms)
+bio_rx_template_array(fido_dev_t *dev, fido_bio_template_array_t *ta, int *ms)
 {
-	const uint8_t	cmd = CTAP_FRAME_INIT | CTAP_CMD_CBOR;
-	unsigned char	reply[2048];
+	unsigned char	reply[FIDO_MAXMSG];
 	int		reply_len;
 	int		r;
 
 	bio_reset_template_array(ta);
 
-	if ((reply_len = fido_rx(dev, cmd, &reply, sizeof(reply), ms)) < 0) {
+	if ((reply_len = fido_rx(dev, CTAP_CMD_CBOR, &reply, sizeof(reply),
+	    ms)) < 0) {
 		fido_log_debug("%s: fido_rx", __func__);
 		return (FIDO_ERR_RX);
 	}
@@ -258,11 +256,11 @@ bio_rx_template_array(fido_dev_t *dev, fido_bio_template_array_t *ta, int ms)
 
 static int
 bio_get_template_array_wait(fido_dev_t *dev, fido_bio_template_array_t *ta,
-    const char *pin, int ms)
+    const char *pin, int *ms)
 {
 	int r;
 
-	if ((r = bio_tx(dev, CMD_ENUM, NULL, 0, pin, NULL)) != FIDO_OK ||
+	if ((r = bio_tx(dev, CMD_ENUM, NULL, 0, pin, NULL, ms)) != FIDO_OK ||
 	    (r = bio_rx_template_array(dev, ta, ms)) != FIDO_OK)
 		return (r);
 
@@ -273,15 +271,17 @@ int
 fido_bio_dev_get_template_array(fido_dev_t *dev, fido_bio_template_array_t *ta,
     const char *pin)
 {
+	int ms = dev->timeout_ms;
+
 	if (pin == NULL)
 		return (FIDO_ERR_INVALID_ARGUMENT);
 
-	return (bio_get_template_array_wait(dev, ta, pin, -1));
+	return (bio_get_template_array_wait(dev, ta, pin, &ms));
 }
 
 static int
 bio_set_template_name_wait(fido_dev_t *dev, const fido_bio_template_t *t,
-    const char *pin, int ms)
+    const char *pin, int *ms)
 {
 	cbor_item_t	*argv[2];
 	int		 r = FIDO_ERR_INTERNAL;
@@ -294,7 +294,8 @@ bio_set_template_name_wait(fido_dev_t *dev, const fido_bio_template_t *t,
 		goto fail;
 	}
 
-	if ((r = bio_tx(dev, CMD_SET_NAME, argv, 2, pin, NULL)) != FIDO_OK ||
+	if ((r = bio_tx(dev, CMD_SET_NAME, argv, 2, pin, NULL,
+	    ms)) != FIDO_OK ||
 	    (r = fido_rx_cbor_status(dev, ms)) != FIDO_OK) {
 		fido_log_debug("%s: tx/rx", __func__);
 		goto fail;
@@ -311,10 +312,12 @@ int
 fido_bio_dev_set_template_name(fido_dev_t *dev, const fido_bio_template_t *t,
     const char *pin)
 {
+	int ms = dev->timeout_ms;
+
 	if (pin == NULL || t->name == NULL)
 		return (FIDO_ERR_INVALID_ARGUMENT);
 
-	return (bio_set_template_name_wait(dev, t, pin, -1));
+	return (bio_set_template_name_wait(dev, t, pin, &ms));
 }
 
 static void
@@ -380,10 +383,9 @@ bio_parse_template_id(const cbor_item_t *key, const cbor_item_t *val,
 
 static int
 bio_rx_enroll_begin(fido_dev_t *dev, fido_bio_template_t *t,
-    fido_bio_enroll_t *e, int ms)
+    fido_bio_enroll_t *e, int *ms)
 {
-	const uint8_t	cmd = CTAP_FRAME_INIT | CTAP_CMD_CBOR;
-	unsigned char	reply[2048];
+	unsigned char	reply[FIDO_MAXMSG];
 	int		reply_len;
 	int		r;
 
@@ -392,7 +394,8 @@ bio_rx_enroll_begin(fido_dev_t *dev, fido_bio_template_t *t,
 	e->remaining_samples = 0;
 	e->last_status = 0;
 
-	if ((reply_len = fido_rx(dev, cmd, &reply, sizeof(reply), ms)) < 0) {
+	if ((reply_len = fido_rx(dev, CTAP_CMD_CBOR, &reply, sizeof(reply),
+	    ms)) < 0) {
 		fido_log_debug("%s: fido_rx", __func__);
 		return (FIDO_ERR_RX);
 	}
@@ -407,13 +410,13 @@ bio_rx_enroll_begin(fido_dev_t *dev, fido_bio_template_t *t,
 		fido_log_debug("%s: bio_parse_template_id", __func__);
 		return (r);
 	}
-    
+
 	return (FIDO_OK);
 }
 
 static int
 bio_enroll_begin_wait(fido_dev_t *dev, fido_bio_template_t *t,
-    fido_bio_enroll_t *e, uint32_t timo_ms, int ms)
+    fido_bio_enroll_t *e, uint32_t timo_ms, int *ms)
 {
 	cbor_item_t	*argv[3];
 	const uint8_t	 cmd = CMD_ENROLL_BEGIN;
@@ -421,12 +424,12 @@ bio_enroll_begin_wait(fido_dev_t *dev, fido_bio_template_t *t,
 
 	memset(&argv, 0, sizeof(argv));
 
-	if ((argv[2] = cbor_build_uint32(timo_ms)) == NULL) {
+	if ((argv[2] = cbor_build_uint(timo_ms)) == NULL) {
 		fido_log_debug("%s: cbor encode", __func__);
 		goto fail;
 	}
 
-	if ((r = bio_tx(dev, cmd, argv, 3, NULL, e->token)) != FIDO_OK ||
+	if ((r = bio_tx(dev, cmd, argv, 3, NULL, e->token, ms)) != FIDO_OK ||
 	    (r = bio_rx_enroll_begin(dev, t, e, ms)) != FIDO_OK) {
 		fido_log_debug("%s: tx/rx", __func__);
 		goto fail;
@@ -446,6 +449,7 @@ fido_bio_dev_enroll_begin(fido_dev_t *dev, fido_bio_template_t *t,
 	es256_pk_t	*pk = NULL;
 	fido_blob_t	*ecdh = NULL;
 	fido_blob_t	*token = NULL;
+	int		 ms = dev->timeout_ms;
 	int		 r;
 
 	if (pin == NULL || e->token != NULL)
@@ -456,13 +460,14 @@ fido_bio_dev_enroll_begin(fido_dev_t *dev, fido_bio_template_t *t,
 		goto fail;
 	}
 
-	if ((r = fido_do_ecdh(dev, &pk, &ecdh)) != FIDO_OK) {
+	if ((r = fido_do_ecdh(dev, &pk, &ecdh, &ms)) != FIDO_OK) {
 		fido_log_debug("%s: fido_do_ecdh", __func__);
 		goto fail;
 	}
 
-	if ((r = fido_dev_get_pin_token(dev, pin, ecdh, pk, token)) != FIDO_OK) {
-		fido_log_debug("%s: fido_dev_get_pin_token", __func__);
+	if ((r = fido_dev_get_uv_token(dev, CTAP_CBOR_BIO_ENROLL_PRE, pin, ecdh,
+	    pk, NULL, token, &ms)) != FIDO_OK) {
+		fido_log_debug("%s: fido_dev_get_uv_token", __func__);
 		goto fail;
 	}
 
@@ -476,21 +481,21 @@ fail:
 	if (r != FIDO_OK)
 		return (r);
 
-	return (bio_enroll_begin_wait(dev, t, e, timo_ms, -1));
+	return (bio_enroll_begin_wait(dev, t, e, timo_ms, &ms));
 }
 
 static int
-bio_rx_enroll_continue(fido_dev_t *dev, fido_bio_enroll_t *e, int ms)
+bio_rx_enroll_continue(fido_dev_t *dev, fido_bio_enroll_t *e, int *ms)
 {
-	const uint8_t	cmd = CTAP_FRAME_INIT | CTAP_CMD_CBOR;
-	unsigned char	reply[2048];
+	unsigned char	reply[FIDO_MAXMSG];
 	int		reply_len;
 	int		r;
 
 	e->remaining_samples = 0;
 	e->last_status = 0;
 
-	if ((reply_len = fido_rx(dev, cmd, &reply, sizeof(reply), ms)) < 0) {
+	if ((reply_len = fido_rx(dev, CTAP_CMD_CBOR, &reply, sizeof(reply),
+	    ms)) < 0) {
 		fido_log_debug("%s: fido_rx", __func__);
 		return (FIDO_ERR_RX);
 	}
@@ -500,13 +505,13 @@ bio_rx_enroll_continue(fido_dev_t *dev, fido_bio_enroll_t *e, int ms)
 		fido_log_debug("%s: bio_parse_enroll_status", __func__);
 		return (r);
 	}
-    
+
 	return (FIDO_OK);
 }
 
 static int
 bio_enroll_continue_wait(fido_dev_t *dev, const fido_bio_template_t *t,
-    fido_bio_enroll_t *e, uint32_t timo_ms, int ms)
+    fido_bio_enroll_t *e, uint32_t timo_ms, int *ms)
 {
 	cbor_item_t	*argv[3];
 	const uint8_t	 cmd = CMD_ENROLL_NEXT;
@@ -515,12 +520,12 @@ bio_enroll_continue_wait(fido_dev_t *dev, const fido_bio_template_t *t,
 	memset(&argv, 0, sizeof(argv));
 
 	if ((argv[0] = fido_blob_encode(&t->id)) == NULL ||
-	    (argv[2] = cbor_build_uint32(timo_ms)) == NULL) {
+	    (argv[2] = cbor_build_uint(timo_ms)) == NULL) {
 		fido_log_debug("%s: cbor encode", __func__);
 		goto fail;
 	}
 
-	if ((r = bio_tx(dev, cmd, argv, 3, NULL, e->token)) != FIDO_OK ||
+	if ((r = bio_tx(dev, cmd, argv, 3, NULL, e->token, ms)) != FIDO_OK ||
 	    (r = bio_rx_enroll_continue(dev, e, ms)) != FIDO_OK) {
 		fido_log_debug("%s: tx/rx", __func__);
 		goto fail;
@@ -537,19 +542,21 @@ int
 fido_bio_dev_enroll_continue(fido_dev_t *dev, const fido_bio_template_t *t,
     fido_bio_enroll_t *e, uint32_t timo_ms)
 {
+	int ms = dev->timeout_ms;
+
 	if (e->token == NULL)
 		return (FIDO_ERR_INVALID_ARGUMENT);
 
-	return (bio_enroll_continue_wait(dev, t, e, timo_ms, -1));
+	return (bio_enroll_continue_wait(dev, t, e, timo_ms, &ms));
 }
 
 static int
-bio_enroll_cancel_wait(fido_dev_t *dev, int ms)
+bio_enroll_cancel_wait(fido_dev_t *dev, int *ms)
 {
 	const uint8_t	cmd = CMD_ENROLL_CANCEL;
 	int		r;
 
-	if ((r = bio_tx(dev, cmd, NULL, 0, NULL, NULL)) != FIDO_OK ||
+	if ((r = bio_tx(dev, cmd, NULL, 0, NULL, NULL, ms)) != FIDO_OK ||
 	    (r = fido_rx_cbor_status(dev, ms)) != FIDO_OK) {
 		fido_log_debug("%s: tx/rx", __func__);
 		return (r);
@@ -561,12 +568,14 @@ bio_enroll_cancel_wait(fido_dev_t *dev, int ms)
 int
 fido_bio_dev_enroll_cancel(fido_dev_t *dev)
 {
-	return (bio_enroll_cancel_wait(dev, -1));
+	int ms = dev->timeout_ms;
+
+	return (bio_enroll_cancel_wait(dev, &ms));
 }
 
 static int
 bio_enroll_remove_wait(fido_dev_t *dev, const fido_bio_template_t *t,
-    const char *pin, int ms)
+    const char *pin, int *ms)
 {
 	cbor_item_t	*argv[1];
 	const uint8_t	 cmd = CMD_ENROLL_REMOVE;
@@ -579,7 +588,7 @@ bio_enroll_remove_wait(fido_dev_t *dev, const fido_bio_template_t *t,
 		goto fail;
 	}
 
-	if ((r = bio_tx(dev, cmd, argv, 1, pin, NULL)) != FIDO_OK ||
+	if ((r = bio_tx(dev, cmd, argv, 1, pin, NULL, ms)) != FIDO_OK ||
 	    (r = fido_rx_cbor_status(dev, ms)) != FIDO_OK) {
 		fido_log_debug("%s: tx/rx", __func__);
 		goto fail;
@@ -596,7 +605,9 @@ int
 fido_bio_dev_enroll_remove(fido_dev_t *dev, const fido_bio_template_t *t,
     const char *pin)
 {
-	return (bio_enroll_remove_wait(dev, t, pin, -1));
+	int ms = dev->timeout_ms;
+
+	return (bio_enroll_remove_wait(dev, t, pin, &ms));
 }
 
 static void
@@ -641,16 +652,16 @@ bio_parse_info(const cbor_item_t *key, const cbor_item_t *val, void *arg)
 }
 
 static int
-bio_rx_info(fido_dev_t *dev, fido_bio_info_t *i, int ms)
+bio_rx_info(fido_dev_t *dev, fido_bio_info_t *i, int *ms)
 {
-	const uint8_t	cmd = CTAP_FRAME_INIT | CTAP_CMD_CBOR;
-	unsigned char	reply[2048];
+	unsigned char	reply[FIDO_MAXMSG];
 	int		reply_len;
 	int		r;
 
 	bio_reset_info(i);
 
-	if ((reply_len = fido_rx(dev, cmd, &reply, sizeof(reply), ms)) < 0) {
+	if ((reply_len = fido_rx(dev, CTAP_CMD_CBOR, &reply, sizeof(reply),
+	    ms)) < 0) {
 		fido_log_debug("%s: fido_rx", __func__);
 		return (FIDO_ERR_RX);
 	}
@@ -665,11 +676,12 @@ bio_rx_info(fido_dev_t *dev, fido_bio_info_t *i, int ms)
 }
 
 static int
-bio_get_info_wait(fido_dev_t *dev, fido_bio_info_t *i, int ms)
+bio_get_info_wait(fido_dev_t *dev, fido_bio_info_t *i, int *ms)
 {
 	int r;
 
-	if ((r = bio_tx(dev, CMD_GET_INFO, NULL, 0, NULL, NULL)) != FIDO_OK ||
+	if ((r = bio_tx(dev, CMD_GET_INFO, NULL, 0, NULL, NULL,
+	    ms)) != FIDO_OK ||
 	    (r = bio_rx_info(dev, i, ms)) != FIDO_OK) {
 		fido_log_debug("%s: tx/rx", __func__);
 		return (r);
@@ -681,7 +693,9 @@ bio_get_info_wait(fido_dev_t *dev, fido_bio_info_t *i, int ms)
 int
 fido_bio_dev_get_info(fido_dev_t *dev, fido_bio_info_t *i)
 {
-	return (bio_get_info_wait(dev, i, -1));
+	int ms = dev->timeout_ms;
+
+	return (bio_get_info_wait(dev, i, &ms));
 }
 
 const char *
@@ -762,9 +776,7 @@ int
 fido_bio_template_set_id(fido_bio_template_t *t, const unsigned char *ptr,
     size_t len)
 {
-	free(t->id.ptr);
-	t->id.ptr = NULL;
-	t->id.len = 0;
+	fido_blob_reset(&t->id);
 
 	if (ptr && fido_blob_set(&t->id, ptr, len) < 0)
 		return (FIDO_ERR_INTERNAL);
