@@ -4,13 +4,7 @@
  * license that can be found in the LICENSE file.
  */
 
-
 #include <openssl/sha.h>
-
-#include <stdint.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "fido.h"
 
 #ifndef TLS
@@ -23,6 +17,7 @@ typedef struct dev_manifest_func_node {
 } dev_manifest_func_node_t;
 
 static TLS dev_manifest_func_node_t *manifest_funcs = NULL;
+static TLS bool disable_u2f_fallback;
 
 static void
 find_manifest_func_node(dev_manifest_func_t f, dev_manifest_func_node_t **curr,
@@ -49,41 +44,71 @@ set_random_report_len(fido_dev_t *dev)
 #endif
 
 static void
-fido_dev_set_flags(fido_dev_t *dev, const fido_cbor_info_t *info)
+fido_dev_set_extension_flags(fido_dev_t *dev, const fido_cbor_info_t *info)
 {
-	char * const	*ptr;
-	const bool	*val;
-	size_t		 len;
-
-	ptr = fido_cbor_info_extensions_ptr(info);
-	len = fido_cbor_info_extensions_len(info);
+	char * const	*ptr = fido_cbor_info_extensions_ptr(info);
+	size_t		 len = fido_cbor_info_extensions_len(info);
 
 	for (size_t i = 0; i < len; i++)
 		if (strcmp(ptr[i], "credProtect") == 0)
 			dev->flags |= FIDO_DEV_CRED_PROT;
+}
 
-	ptr = fido_cbor_info_options_name_ptr(info);
-	val = fido_cbor_info_options_value_ptr(info);
-	len = fido_cbor_info_options_len(info);
+static void
+fido_dev_set_option_flags(fido_dev_t *dev, const fido_cbor_info_t *info)
+{
+	char * const	*ptr = fido_cbor_info_options_name_ptr(info);
+	const bool	*val = fido_cbor_info_options_value_ptr(info);
+	size_t		 len = fido_cbor_info_options_len(info);
 
 	for (size_t i = 0; i < len; i++)
 		if (strcmp(ptr[i], "clientPin") == 0) {
-			if (val[i] == true)
-				dev->flags |= FIDO_DEV_PIN_SET;
-			else
-				dev->flags |= FIDO_DEV_PIN_UNSET;
+			dev->flags |= val[i] ? FIDO_DEV_PIN_SET : FIDO_DEV_PIN_UNSET;
 		} else if (strcmp(ptr[i], "credMgmt") == 0 ||
 			   strcmp(ptr[i], "credentialMgmtPreview") == 0) {
-			if (val[i] == true)
+			if (val[i])
 				dev->flags |= FIDO_DEV_CREDMAN;
+		} else if (strcmp(ptr[i], "uv") == 0) {
+			dev->flags |= val[i] ? FIDO_DEV_UV_SET : FIDO_DEV_UV_UNSET;
+		} else if (strcmp(ptr[i], "pinUvAuthToken") == 0) {
+			if (val[i])
+				dev->flags |= FIDO_DEV_TOKEN_PERMS;
 		}
 }
 
-static int
-fido_dev_open_tx(fido_dev_t *dev, const char *path)
+static void
+fido_dev_set_protocol_flags(fido_dev_t *dev, const fido_cbor_info_t *info)
 {
-	const uint8_t	cmd = CTAP_CMD_INIT;
-	int		r;
+	const uint8_t	*ptr = fido_cbor_info_protocols_ptr(info);
+	size_t		 len = fido_cbor_info_protocols_len(info);
+
+	for (size_t i = 0; i < len; i++)
+		switch (ptr[i]) {
+		case CTAP_PIN_PROTOCOL1:
+			dev->flags |= FIDO_DEV_PIN_PROTOCOL1;
+			break;
+		case CTAP_PIN_PROTOCOL2:
+			dev->flags |= FIDO_DEV_PIN_PROTOCOL2;
+			break;
+		default:
+			fido_log_debug("%s: unknown protocol %u", __func__,
+			    ptr[i]);
+			break;
+		}
+}
+
+static void
+fido_dev_set_flags(fido_dev_t *dev, const fido_cbor_info_t *info)
+{
+	fido_dev_set_extension_flags(dev, info);
+	fido_dev_set_option_flags(dev, info);
+	fido_dev_set_protocol_flags(dev, info);
+}
+
+static int
+fido_dev_open_tx(fido_dev_t *dev, const char *path, int *ms)
+{
+	int r;
 
 	if (dev->io_handle != NULL) {
 		fido_log_debug("%s: handle=%p", __func__, dev->io_handle);
@@ -136,7 +161,8 @@ fido_dev_open_tx(fido_dev_t *dev, const char *path)
 		goto fail;
 	}
 
-	if (fido_tx(dev, cmd, &dev->nonce, sizeof(dev->nonce)) < 0) {
+	if (fido_tx(dev, CTAP_CMD_INIT, &dev->nonce, sizeof(dev->nonce),
+	    ms) < 0) {
 		fido_log_debug("%s: fido_tx", __func__);
 		r = FIDO_ERR_TX;
 		goto fail;
@@ -151,7 +177,7 @@ fail:
 }
 
 static int
-fido_dev_open_rx(fido_dev_t *dev, int ms)
+fido_dev_open_rx(fido_dev_t *dev, int *ms)
 {
 	fido_cbor_info_t	*info = NULL;
 	int			 reply_len;
@@ -184,7 +210,12 @@ fido_dev_open_rx(fido_dev_t *dev, int ms)
 			r = FIDO_ERR_INTERNAL;
 			goto fail;
 		}
-		if (fido_dev_get_cbor_info_wait(dev, info, ms) != FIDO_OK) {
+		if ((r = fido_dev_get_cbor_info_wait(dev, info,
+		    ms)) != FIDO_OK) {
+			fido_log_debug("%s: fido_dev_cbor_info_wait: %d",
+			    __func__, r);
+			if (disable_u2f_fallback)
+				goto fail;
 			fido_log_debug("%s: falling back to u2f", __func__);
 			fido_dev_force_u2f(dev);
 		} else {
@@ -193,8 +224,9 @@ fido_dev_open_rx(fido_dev_t *dev, int ms)
 	}
 
 	if (fido_dev_is_fido2(dev) && info != NULL) {
+		dev->maxmsgsize = fido_cbor_info_maxmsgsiz(info);
 		fido_log_debug("%s: FIDO_MAXMSG=%d, maxmsgsiz=%lu", __func__,
-		    FIDO_MAXMSG, (unsigned long)fido_cbor_info_maxmsgsiz(info));
+		    FIDO_MAXMSG, (unsigned long)dev->maxmsgsize);
 	}
 
 	r = FIDO_OK;
@@ -210,11 +242,15 @@ fail:
 }
 
 static int
-fido_dev_open_wait(fido_dev_t *dev, const char *path, int ms)
+fido_dev_open_wait(fido_dev_t *dev, const char *path, int *ms)
 {
 	int r;
 
-	if ((r = fido_dev_open_tx(dev, path)) != FIDO_OK ||
+#ifdef USE_WINHELLO
+	if (strcmp(path, FIDO_WINHELLO_PATH) == 0)
+		return (fido_winhello_open(dev));
+#endif
+	if ((r = fido_dev_open_tx(dev, path, ms)) != FIDO_OK ||
 	    (r = fido_dev_open_rx(dev, ms)) != FIDO_OK)
 		return (r);
 
@@ -270,6 +306,14 @@ fido_dev_info_manifest(fido_dev_info_t *devlist, size_t ilen, size_t *olen)
 
 	if (fido_dev_register_manifest_func(fido_hid_manifest) != FIDO_OK)
 		return (FIDO_ERR_INTERNAL);
+#ifdef NFC_LINUX
+	if (fido_dev_register_manifest_func(fido_nfc_manifest) != FIDO_OK)
+		return (FIDO_ERR_INTERNAL);
+#endif
+#ifdef USE_WINHELLO
+	if (fido_dev_register_manifest_func(fido_winhello_manifest) != FIDO_OK)
+		return (FIDO_ERR_INTERNAL);
+#endif
 
 	for (curr = manifest_funcs; curr != NULL; curr = curr->next) {
 		curr_olen = 0;
@@ -288,21 +332,45 @@ fido_dev_info_manifest(fido_dev_info_t *devlist, size_t ilen, size_t *olen)
 int
 fido_dev_open_with_info(fido_dev_t *dev)
 {
+	int ms = dev->timeout_ms;
+
 	if (dev->path == NULL)
 		return (FIDO_ERR_INVALID_ARGUMENT);
 
-	return (fido_dev_open_wait(dev, dev->path, -1));
+	return (fido_dev_open_wait(dev, dev->path, &ms));
 }
 
 int
 fido_dev_open(fido_dev_t *dev, const char *path)
 {
-	return (fido_dev_open_wait(dev, path, -1));
+	int ms = dev->timeout_ms;
+
+#ifdef NFC_LINUX
+	if (strncmp(path, FIDO_NFC_PREFIX, strlen(FIDO_NFC_PREFIX)) == 0) {
+		dev->io_own = true;
+		dev->io = (fido_dev_io_t) {
+			fido_nfc_open,
+			fido_nfc_close,
+			fido_nfc_read,
+			fido_nfc_write,
+		};
+		dev->transport = (fido_dev_transport_t) {
+			fido_nfc_rx,
+			fido_nfc_tx,
+		};
+	}
+#endif
+
+	return (fido_dev_open_wait(dev, path, &ms));
 }
 
 int
 fido_dev_close(fido_dev_t *dev)
 {
+#ifdef USE_WINHELLO
+	if (dev->flags & FIDO_DEV_WINHELLO)
+		return (fido_winhello_close(dev));
+#endif
 	if (dev->io_handle == NULL || dev->io.close == NULL)
 		return (FIDO_ERR_INVALID_ARGUMENT);
 
@@ -314,12 +382,33 @@ fido_dev_close(fido_dev_t *dev)
 }
 
 int
-fido_dev_cancel(fido_dev_t *dev)
+fido_dev_set_sigmask(fido_dev_t *dev, const fido_sigset_t *sigmask)
 {
-	if (fido_dev_is_fido2(dev) == false)
+	if (dev->io_handle == NULL || sigmask == NULL)
 		return (FIDO_ERR_INVALID_ARGUMENT);
 
-	if (fido_tx(dev, CTAP_CMD_CANCEL, NULL, 0) < 0)
+#ifdef NFC_LINUX
+	if (dev->transport.rx == fido_nfc_rx && dev->io.read == fido_nfc_read)
+		return (fido_nfc_set_sigmask(dev->io_handle, sigmask));
+#endif
+	if (dev->transport.rx == NULL && dev->io.read == fido_hid_read)
+		return (fido_hid_set_sigmask(dev->io_handle, sigmask));
+
+	return (FIDO_ERR_INVALID_ARGUMENT);
+}
+
+int
+fido_dev_cancel(fido_dev_t *dev)
+{
+	int ms = dev->timeout_ms;
+
+#ifdef USE_WINHELLO
+	if (dev->flags & FIDO_DEV_WINHELLO)
+		return (fido_winhello_cancel(dev));
+#endif
+	if (fido_dev_is_fido2(dev) == false)
+		return (FIDO_ERR_INVALID_ARGUMENT);
+	if (fido_tx(dev, CTAP_CMD_CANCEL, NULL, 0, &ms) < 0)
 		return (FIDO_ERR_TX);
 
 	return (FIDO_OK);
@@ -335,6 +424,7 @@ fido_dev_get_touch_begin(fido_dev_t *dev)
 	unsigned char	 cdh[SHA256_DIGEST_LENGTH];
 	fido_rp_t	 rp;
 	fido_user_t	 user;
+	int		 ms = dev->timeout_ms;
 	int		 r = FIDO_ERR_INTERNAL;
 
 	memset(&f, 0, sizeof(f));
@@ -344,7 +434,7 @@ fido_dev_get_touch_begin(fido_dev_t *dev)
 	memset(&user, 0, sizeof(user));
 
 	if (fido_dev_is_fido2(dev) == false)
-		return (u2f_get_touch_begin(dev));
+		return (u2f_get_touch_begin(dev, &ms));
 
 	if (SHA256((const void *)clientdata, strlen(clientdata), cdh) != cdh) {
 		fido_log_debug("%s: sha256", __func__);
@@ -372,14 +462,14 @@ fido_dev_get_touch_begin(fido_dev_t *dev)
 
 	if (fido_dev_supports_pin(dev)) {
 		if ((argv[7] = cbor_new_definite_bytestring()) == NULL ||
-		    (argv[8] = cbor_encode_pin_opt()) == NULL) {
+		    (argv[8] = cbor_encode_pin_opt(dev)) == NULL) {
 			fido_log_debug("%s: cbor encode", __func__);
 			goto fail;
 		}
 	}
 
 	if (cbor_build_frame(CTAP_CBOR_MAKECRED, argv, nitems(argv), &f) < 0 ||
-	    fido_tx(dev, CTAP_CMD_CBOR, f.ptr, f.len) < 0) {
+	    fido_tx(dev, CTAP_CMD_CBOR, f.ptr, f.len, &ms) < 0) {
 		fido_log_debug("%s: fido_tx", __func__);
 		r = FIDO_ERR_TX;
 		goto fail;
@@ -404,9 +494,9 @@ fido_dev_get_touch_status(fido_dev_t *dev, int *touched, int ms)
 	*touched = 0;
 
 	if (fido_dev_is_fido2(dev) == false)
-		return (u2f_get_touch_status(dev, touched, ms));
+		return (u2f_get_touch_status(dev, touched, &ms));
 
-	switch ((r = fido_rx_cbor_status(dev, ms))) {
+	switch ((r = fido_rx_cbor_status(dev, &ms))) {
 	case FIDO_ERR_PIN_AUTH_INVALID:
 	case FIDO_ERR_PIN_INVALID:
 	case FIDO_ERR_PIN_NOT_SET:
@@ -458,11 +548,20 @@ fido_dev_set_transport_functions(fido_dev_t *dev, const fido_dev_transport_t *t)
 	return (FIDO_OK);
 }
 
+void *
+fido_dev_io_handle(const fido_dev_t *dev)
+{
+
+	return (dev->io_handle);
+}
+
 void
 fido_init(int flags)
 {
 	if (flags & FIDO_DEBUG || getenv("FIDO_DEBUG") != NULL)
 		fido_log_init();
+
+	disable_u2f_fallback = (flags & FIDO_DISABLE_U2F_FALLBACK);
 }
 
 fido_dev_t *
@@ -474,6 +573,7 @@ fido_dev_new(void)
 		return (NULL);
 
 	dev->cid = CTAP_CID_BROADCAST;
+	dev->timeout_ms = -1;
 	dev->io = (fido_dev_io_t) {
 		&fido_hid_open,
 		&fido_hid_close,
@@ -492,17 +592,20 @@ fido_dev_new_with_info(const fido_dev_info_t *di)
 	if ((dev = calloc(1, sizeof(*dev))) == NULL)
 		return (NULL);
 
+#if 0
 	if (di->io.open == NULL || di->io.close == NULL ||
 	    di->io.read == NULL || di->io.write == NULL) {
 		fido_log_debug("%s: NULL function", __func__);
 		fido_dev_free(&dev);
 		return (NULL);
 	}
+#endif
 
 	dev->io = di->io;
 	dev->io_own = di->transport.tx != NULL || di->transport.rx != NULL;
 	dev->transport = di->transport;
 	dev->cid = CTAP_CID_BROADCAST;
+	dev->timeout_ms = -1;
 
 	if ((dev->path = strdup(di->path)) == NULL) {
 		fido_log_debug("%s: strdup", __func__);
@@ -563,6 +666,12 @@ fido_dev_is_fido2(const fido_dev_t *dev)
 	return (dev->attr.flags & FIDO_CAP_CBOR);
 }
 
+bool
+fido_dev_is_winhello(const fido_dev_t *dev)
+{
+	return (dev->flags & FIDO_DEV_WINHELLO);
+}
+
 bool
 fido_dev_supports_pin(const fido_dev_t *dev)
 {
@@ -587,6 +696,24 @@ fido_dev_supports_credman(const fido_dev_t *dev)
 	return (dev->flags & FIDO_DEV_CREDMAN);
 }
 
+bool
+fido_dev_supports_uv(const fido_dev_t *dev)
+{
+	return (dev->flags & (FIDO_DEV_UV_SET|FIDO_DEV_UV_UNSET));
+}
+
+bool
+fido_dev_has_uv(const fido_dev_t *dev)
+{
+	return (dev->flags & FIDO_DEV_UV_SET);
+}
+
+bool
+fido_dev_supports_permissions(const fido_dev_t *dev)
+{
+	return (dev->flags & FIDO_DEV_TOKEN_PERMS);
+}
+
 void
 fido_dev_force_u2f(fido_dev_t *dev)
 {
@@ -599,3 +726,31 @@ fido_dev_force_fido2(fido_dev_t *dev)
 {
 	dev->attr.flags |= FIDO_CAP_CBOR;
 }
+
+uint8_t
+fido_dev_get_pin_protocol(const fido_dev_t *dev)
+{
+	if (dev->flags & FIDO_DEV_PIN_PROTOCOL2)
+		return (CTAP_PIN_PROTOCOL2);
+	else if (dev->flags & FIDO_DEV_PIN_PROTOCOL1)
+		return (CTAP_PIN_PROTOCOL1);
+
+	return (0);
+}
+
+uint64_t
+fido_dev_maxmsgsize(const fido_dev_t *dev)
+{
+	return (dev->maxmsgsize);
+}
+
+int
+fido_dev_set_timeout(fido_dev_t *dev, int ms)
+{
+	if (ms < -1)
+		return (FIDO_ERR_INVALID_ARGUMENT);
+
+	dev->timeout_ms = ms;
+
+	return (FIDO_OK);
+}
