@@ -8,24 +8,23 @@
 
 #include <sys/ioctl.h>
 #include <dev/usb/usb.h>
-#include <dev/usb/usbhid.h>
 
 #include <errno.h>
 #include <fcntl.h>
-#include <string.h>
-#include <unistd.h>
-#include <usbhid.h>
 #include <poll.h>
+#include <signal.h>
+#include <unistd.h>
 
 #include "fido.h"
 
 #define MAX_UHID	64
-#define MAX_REPORT_LEN	(sizeof(((struct usb_ctl_report *)(NULL))->ucr_data))
 
 struct hid_openbsd {
 	int fd;
 	size_t report_in_len;
 	size_t report_out_len;
+	sigset_t sigmask;
+	const sigset_t *sigmaskp;
 };
 
 int
@@ -33,11 +32,8 @@ fido_hid_manifest(fido_dev_info_t *devlist, size_t ilen, size_t *olen)
 {
 	size_t i;
 	char path[64];
-	int is_fido, fd;
+	int fd;
 	struct usb_device_info udi;
-	report_desc_t rdesc = NULL;
-	hid_data_t hdata = NULL;
-	hid_item_t hitem;
 	fido_dev_info_t *di;
 
 	if (ilen == 0)
@@ -47,49 +43,19 @@ fido_hid_manifest(fido_dev_info_t *devlist, size_t ilen, size_t *olen)
 		return (FIDO_ERR_INVALID_ARGUMENT);
 
 	for (i = *olen = 0; i < MAX_UHID && *olen < ilen; i++) {
-		snprintf(path, sizeof(path), "/dev/uhid%zu", i);
-		if ((fd = open(path, O_RDWR)) == -1) {
-			if (errno != ENOENT && errno != ENXIO) {
-				fido_log_debug("%s: open %s: %s", __func__,
-				    path, strerror(errno));
-			}
+		snprintf(path, sizeof(path), "/dev/fido/%zu", i);
+		if ((fd = fido_hid_unix_open(path)) == -1)
 			continue;
-		}
 		memset(&udi, 0, sizeof(udi));
-		if (ioctl(fd, USB_GET_DEVICEINFO, &udi) != 0) {
-			fido_log_debug("%s: get device info %s: %s", __func__,
-			    path, strerror(errno));
-			close(fd);
-			continue;
-		}
-		if ((rdesc = hid_get_report_desc(fd)) == NULL) {
-			fido_log_debug("%s: failed to get report descriptor: %s",
+		if (ioctl(fd, IOCTL_REQ(USB_GET_DEVICEINFO), &udi) == -1) {
+			fido_log_error(errno, "%s: get device info %s",
 			    __func__, path);
-			close(fd);
+			if (close(fd) == -1)
+				fido_log_error(errno, "%s: close", __func__);
 			continue;
 		}
-		if ((hdata = hid_start_parse(rdesc,
-		    1<<hid_collection, -1)) == NULL) {
-			fido_log_debug("%s: failed to parse report descriptor: %s",
-			    __func__, path);
-			hid_dispose_report_desc(rdesc);
-			close(fd);
-			continue;
-		}
-		is_fido = 0;
-		for (is_fido = 0; !is_fido;) {
-			memset(&hitem, 0, sizeof(hitem));
-			if (hid_get_item(hdata, &hitem) <= 0)
-				break;
-			if ((hitem._usage_page & 0xFFFF0000) == 0xf1d00000)
-				is_fido = 1;
-		}
-		hid_end_parse(hdata);
-		hid_dispose_report_desc(rdesc);
-		close(fd);
-
-		if (!is_fido)
-			continue;
+		if (close(fd) == -1)
+			fido_log_error(errno, "%s: close", __func__);
 
 		fido_log_debug("%s: %s: bus = 0x%02x, addr = 0x%02x",
 		    __func__, path, udi.udi_bus, udi.udi_addr);
@@ -116,8 +82,8 @@ fido_hid_manifest(fido_dev_info_t *devlist, size_t ilen, size_t *olen)
 			explicit_bzero(di, sizeof(*di));
 			return FIDO_ERR_INTERNAL;
 		}
-		di->vendor_id = udi.udi_vendorNo;
-		di->product_id = udi.udi_productNo;
+		di->vendor_id = (int16_t)udi.udi_vendorNo;
+		di->product_id = (int16_t)udi.udi_productNo;
 		(*olen)++;
 	}
 
@@ -159,7 +125,7 @@ terrible_ping_kludge(struct hid_openbsd *ctx)
 		pfd.fd = ctx->fd;
 		pfd.events = POLLIN;
 		if ((n = poll(&pfd, 1, 100)) == -1) {
-			fido_log_debug("%s: poll: %s", __func__, strerror(errno));
+			fido_log_error(errno, "%s: poll", __func__);
 			return -1;
 		} else if (n == 0) {
 			fido_log_debug("%s: timed out", __func__);
@@ -172,8 +138,8 @@ terrible_ping_kludge(struct hid_openbsd *ctx)
 		 * so we might get an error, but we don't care - we're
 		 * synched now.
 		 */
-		fido_log_debug("%s: got reply", __func__);
-		fido_log_xxd(data, ctx->report_out_len);
+		fido_log_xxd(data, ctx->report_out_len, "%s: got reply",
+		    __func__);
 		return 0;
 	}
 	fido_log_debug("%s: no response", __func__);
@@ -184,42 +150,15 @@ void *
 fido_hid_open(const char *path)
 {
 	struct hid_openbsd *ret = NULL;
-	report_desc_t rdesc = NULL;
-	int len, usb_report_id = 0;
 
 	if ((ret = calloc(1, sizeof(*ret))) == NULL ||
-	    (ret->fd = open(path, O_RDWR)) < 0) {
+	    (ret->fd = fido_hid_unix_open(path)) == -1) {
 		free(ret);
 		return (NULL);
 	}
-	if (ioctl(ret->fd, USB_GET_REPORT_ID, &usb_report_id) != 0) {
-		fido_log_debug("%s: failed to get report ID: %s", __func__,
-		    strerror(errno));
-		goto fail;
-	}
-	if ((rdesc = hid_get_report_desc(ret->fd)) == NULL) {
-		fido_log_debug("%s: failed to get report descriptor", __func__);
-		goto fail;
-	}
-	if ((len = hid_report_size(rdesc, hid_input, usb_report_id)) <= 0 ||
-	    (size_t)len > MAX_REPORT_LEN) {
-		fido_log_debug("%s: bad input report size %d", __func__, len);
-		goto fail;
-	}
-	ret->report_in_len = (size_t)len;
-	if ((len = hid_report_size(rdesc, hid_output, usb_report_id)) <= 0 ||
-	    (size_t)len > MAX_REPORT_LEN) {
-		fido_log_debug("%s: bad output report size %d", __func__, len);
- fail:
-		hid_dispose_report_desc(rdesc);
-		close(ret->fd);
-		free(ret);
-		return NULL;
-	}	
-	ret->report_out_len = (size_t)len;
-	hid_dispose_report_desc(rdesc);
-	fido_log_debug("%s: USB report ID %d, inlen = %zu outlen = %zu",
-	    __func__, usb_report_id, ret->report_in_len, ret->report_out_len);
+	ret->report_in_len = ret->report_out_len = CTAP_MAX_REPORT_LEN;
+	fido_log_debug("%s: inlen = %zu outlen = %zu", __func__,
+	    ret->report_in_len, ret->report_out_len);
 
 	/*
 	 * OpenBSD (as of 201910) has a bug that causes it to lose
@@ -239,27 +178,50 @@ fido_hid_close(void *handle)
 {
 	struct hid_openbsd *ctx = (struct hid_openbsd *)handle;
 
-	close(ctx->fd);
+	if (close(ctx->fd) == -1)
+		fido_log_error(errno, "%s: close", __func__);
+
 	free(ctx);
 }
 
+int
+fido_hid_set_sigmask(void *handle, const fido_sigset_t *sigmask)
+{
+	struct hid_openbsd *ctx = handle;
+
+	ctx->sigmask = *sigmask;
+	ctx->sigmaskp = &ctx->sigmask;
+
+	return (FIDO_OK);
+}
+
 int
 fido_hid_read(void *handle, unsigned char *buf, size_t len, int ms)
 {
 	struct hid_openbsd *ctx = (struct hid_openbsd *)handle;
 	ssize_t r;
 
-	(void)ms; /* XXX */
-
 	if (len != ctx->report_in_len) {
 		fido_log_debug("%s: invalid len: got %zu, want %zu", __func__,
 		    len, ctx->report_in_len);
 		return (-1);
 	}
-	if ((r = read(ctx->fd, buf, len)) == -1 || (size_t)r != len) {
-		fido_log_debug("%s: read: %s", __func__, strerror(errno));
+
+	if (fido_hid_unix_wait(ctx->fd, ms, ctx->sigmaskp) < 0) {
+		fido_log_debug("%s: fd not ready", __func__);
 		return (-1);
 	}
+
+	if ((r = read(ctx->fd, buf, len)) == -1) {
+		fido_log_error(errno, "%s: read", __func__);
+		return (-1);
+	}
+
+	if (r < 0 || (size_t)r != len) {
+		fido_log_debug("%s: %zd != %zu", __func__, r, len);
+		return (-1);
+	}
+
 	return ((int)len);
 }
 
@@ -274,10 +236,32 @@ fido_hid_write(void *handle, const unsigned char *buf, size_t len)
 		    len, ctx->report_out_len);
 		return (-1);
 	}
-	if ((r = write(ctx->fd, buf + 1, len - 1)) == -1 ||
-	    (size_t)r != len - 1) {
-		fido_log_debug("%s: write: %s", __func__, strerror(errno));
+
+	if ((r = write(ctx->fd, buf + 1, len - 1)) == -1) {
+		fido_log_error(errno, "%s: write", __func__);
 		return (-1);
 	}
+
+	if (r < 0 || (size_t)r != len - 1) {
+		fido_log_debug("%s: %zd != %zu", __func__, r, len - 1);
+		return (-1);
+	}
+
 	return ((int)len);
 }
+
+size_t
+fido_hid_report_in_len(void *handle)
+{
+	struct hid_openbsd *ctx = handle;
+
+	return (ctx->report_in_len);
+}
+
+size_t
+fido_hid_report_out_len(void *handle)
+{
+	struct hid_openbsd *ctx = handle;
+
+	return (ctx->report_out_len);
+}
