@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018 Yubico AB. All rights reserved.
+ * Copyright (c) 2018-2021 Yubico AB. All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the LICENSE file.
  */
@@ -7,14 +7,16 @@
 #include <openssl/sha.h>
 #include <openssl/x509.h>
 
-#include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include <errno.h>
 
 #include "fido.h"
 #include "fido/es256.h"
 
+#define U2F_PACE_MS (100)
+
 #if defined(_MSC_VER)
 static int
 usleep(unsigned int usec)
@@ -25,6 +27,28 @@ usleep(unsigned int usec)
 }
 #endif
 
+static int
+delay_ms(unsigned int ms, int *ms_remain)
+{
+	if (*ms_remain > -1 && (unsigned int)*ms_remain < ms)
+		ms = (unsigned int)*ms_remain;
+
+	if (ms > UINT_MAX / 1000) {
+		fido_log_debug("%s: ms=%u", __func__, ms);
+		return (-1);
+	}
+
+	if (usleep(ms * 1000) < 0) {
+		fido_log_error(errno, "%s: usleep", __func__);
+		return (-1);
+	}
+
+	if (*ms_remain > -1)
+		*ms_remain -= (int)ms;
+
+	return (0);
+}
+
 static int
 sig_get(fido_blob_t *sig, const unsigned char **buf, size_t *len)
 {
@@ -32,13 +56,8 @@ sig_get(fido_blob_t *sig, const unsigned char **buf, size_t *len)
 	if ((sig->ptr = calloc(1, sig->len)) == NULL ||
 	    fido_buf_read(buf, len, sig->ptr, sig->len) < 0) {
 		fido_log_debug("%s: fido_buf_read", __func__);
-		if (sig->ptr != NULL) {
-			explicit_bzero(sig->ptr, sig->len);
-			free(sig->ptr);
-			sig->ptr = NULL;
-			sig->len = 0;
-			return (-1);
-		}
+		fido_blob_reset(sig);
+		return (-1);
 	}
 
 	return (0);
@@ -75,11 +94,8 @@ fail:
 	if (cert != NULL)
 		X509_free(cert);
 
-	if (ok < 0) {
-		free(x5c->ptr);
-		x5c->ptr = NULL;
-		x5c->len = 0;
-	}
+	if (ok < 0)
+		fido_blob_reset(x5c);
 
 	return (ok);
 }
@@ -122,25 +138,21 @@ authdata_fake(const char *rp_id, uint8_t flags, uint32_t sigcount,
 	return (0);
 }
 
+/* TODO: use u2f_get_touch_begin & u2f_get_touch_status instead */
 static int
-send_dummy_register(fido_dev_t *dev, int ms)
+send_dummy_register(fido_dev_t *dev, int *ms)
 {
-	const uint8_t	 cmd = CTAP_FRAME_INIT | CTAP_CMD_MSG;
 	iso7816_apdu_t	*apdu = NULL;
 	unsigned char	 challenge[SHA256_DIGEST_LENGTH];
 	unsigned char	 application[SHA256_DIGEST_LENGTH];
-	unsigned char	 reply[2048];
+	unsigned char	 reply[FIDO_MAXMSG];
 	int		 r;
 
-#ifdef FIDO_FUZZ
-	ms = 0; /* XXX */
-#endif
-
 	/* dummy challenge & application */
 	memset(&challenge, 0xff, sizeof(challenge));
 	memset(&application, 0xff, sizeof(application));
 
-	if ((apdu = iso7816_new(U2F_CMD_REGISTER, 0, 2 *
+	if ((apdu = iso7816_new(0, U2F_CMD_REGISTER, 0, 2 *
 	    SHA256_DIGEST_LENGTH)) == NULL ||
 	    iso7816_add(apdu, &challenge, sizeof(challenge)) < 0 ||
 	    iso7816_add(apdu, &application, sizeof(application)) < 0) {
@@ -150,19 +162,19 @@ send_dummy_register(fido_dev_t *dev, int ms)
 	}
 
 	do {
-		if (fido_tx(dev, cmd, iso7816_ptr(apdu),
-		    iso7816_len(apdu)) < 0) {
+		if (fido_tx(dev, CTAP_CMD_MSG, iso7816_ptr(apdu),
+		    iso7816_len(apdu), ms) < 0) {
 			fido_log_debug("%s: fido_tx", __func__);
 			r = FIDO_ERR_TX;
 			goto fail;
 		}
-		if (fido_rx(dev, cmd, &reply, sizeof(reply), ms) < 2) {
+		if (fido_rx(dev, CTAP_CMD_MSG, &reply, sizeof(reply), ms) < 2) {
 			fido_log_debug("%s: fido_rx", __func__);
 			r = FIDO_ERR_RX;
 			goto fail;
 		}
-		if (usleep((ms == -1 ? 100 : ms) * 1000) < 0) {
-			fido_log_debug("%s: usleep", __func__);
+		if (delay_ms(U2F_PACE_MS, ms) != 0) {
+			fido_log_debug("%s: delay_ms", __func__);
 			r = FIDO_ERR_RX;
 			goto fail;
 		}
@@ -177,13 +189,12 @@ fail:
 
 static int
 key_lookup(fido_dev_t *dev, const char *rp_id, const fido_blob_t *key_id,
-    int *found, int ms)
+    int *found, int *ms)
 {
-	const uint8_t	 cmd = CTAP_FRAME_INIT | CTAP_CMD_MSG;
 	iso7816_apdu_t	*apdu = NULL;
 	unsigned char	 challenge[SHA256_DIGEST_LENGTH];
 	unsigned char	 rp_id_hash[SHA256_DIGEST_LENGTH];
-	unsigned char	 reply[8];
+	unsigned char	 reply[FIDO_MAXMSG];
 	uint8_t		 key_id_len;
 	int		 r;
 
@@ -206,8 +217,8 @@ key_lookup(fido_dev_t *dev, const char *rp_id, const fido_blob_t *key_id,
 
 	key_id_len = (uint8_t)key_id->len;
 
-	if ((apdu = iso7816_new(U2F_CMD_AUTH, U2F_AUTH_CHECK, 2 *
-	    SHA256_DIGEST_LENGTH + sizeof(key_id_len) + key_id_len)) == NULL ||
+	if ((apdu = iso7816_new(0, U2F_CMD_AUTH, U2F_AUTH_CHECK, (uint16_t)(2 *
+	    SHA256_DIGEST_LENGTH + sizeof(key_id_len) + key_id_len))) == NULL ||
 	    iso7816_add(apdu, &challenge, sizeof(challenge)) < 0 ||
 	    iso7816_add(apdu, &rp_id_hash, sizeof(rp_id_hash)) < 0 ||
 	    iso7816_add(apdu, &key_id_len, sizeof(key_id_len)) < 0 ||
@@ -217,12 +228,13 @@ key_lookup(fido_dev_t *dev, const char *rp_id, const fido_blob_t *key_id,
 		goto fail;
 	}
 
-	if (fido_tx(dev, cmd, iso7816_ptr(apdu), iso7816_len(apdu)) < 0) {
+	if (fido_tx(dev, CTAP_CMD_MSG, iso7816_ptr(apdu),
+	    iso7816_len(apdu), ms) < 0) {
 		fido_log_debug("%s: fido_tx", __func__);
 		r = FIDO_ERR_TX;
 		goto fail;
 	}
-	if (fido_rx(dev, cmd, &reply, sizeof(reply), ms) != 2) {
+	if (fido_rx(dev, CTAP_CMD_MSG, &reply, sizeof(reply), ms) != 2) {
 		fido_log_debug("%s: fido_rx", __func__);
 		r = FIDO_ERR_RX;
 		goto fail;
@@ -283,18 +295,17 @@ parse_auth_reply(fido_blob_t *sig, fido_blob_t *ad, const char *rp_id,
 
 static int
 do_auth(fido_dev_t *dev, const fido_blob_t *cdh, const char *rp_id,
-    const fido_blob_t *key_id, fido_blob_t *sig, fido_blob_t *ad, int ms)
+    const fido_blob_t *key_id, fido_blob_t *sig, fido_blob_t *ad, int *ms)
 {
-	const uint8_t	 cmd = CTAP_FRAME_INIT | CTAP_CMD_MSG;
 	iso7816_apdu_t	*apdu = NULL;
 	unsigned char	 rp_id_hash[SHA256_DIGEST_LENGTH];
-	unsigned char	 reply[128];
+	unsigned char	 reply[FIDO_MAXMSG];
 	int		 reply_len;
 	uint8_t		 key_id_len;
 	int		 r;
 
 #ifdef FIDO_FUZZ
-	ms = 0; /* XXX */
+	*ms = 0; /* XXX */
 #endif
 
 	if (cdh->len != SHA256_DIGEST_LENGTH || key_id->len > UINT8_MAX ||
@@ -314,8 +325,8 @@ do_auth(fido_dev_t *dev, const fido_blob_t *cdh, const char *rp_id,
 
 	key_id_len = (uint8_t)key_id->len;
 
-	if ((apdu = iso7816_new(U2F_CMD_AUTH, U2F_AUTH_SIGN, 2 *
-	    SHA256_DIGEST_LENGTH + sizeof(key_id_len) + key_id_len)) == NULL ||
+	if ((apdu = iso7816_new(0, U2F_CMD_AUTH, U2F_AUTH_SIGN, (uint16_t)(2 *
+	    SHA256_DIGEST_LENGTH + sizeof(key_id_len) + key_id_len))) == NULL ||
 	    iso7816_add(apdu, cdh->ptr, cdh->len) < 0 ||
 	    iso7816_add(apdu, &rp_id_hash, sizeof(rp_id_hash)) < 0 ||
 	    iso7816_add(apdu, &key_id_len, sizeof(key_id_len)) < 0 ||
@@ -326,20 +337,20 @@ do_auth(fido_dev_t *dev, const fido_blob_t *cdh, const char *rp_id,
 	}
 
 	do {
-		if (fido_tx(dev, cmd, iso7816_ptr(apdu),
-		    iso7816_len(apdu)) < 0) {
+		if (fido_tx(dev, CTAP_CMD_MSG, iso7816_ptr(apdu),
+		    iso7816_len(apdu), ms) < 0) {
 			fido_log_debug("%s: fido_tx", __func__);
 			r = FIDO_ERR_TX;
 			goto fail;
 		}
-		if ((reply_len = fido_rx(dev, cmd, &reply, sizeof(reply),
-		    ms)) < 2) {
+		if ((reply_len = fido_rx(dev, CTAP_CMD_MSG, &reply,
+		    sizeof(reply), ms)) < 2) {
 			fido_log_debug("%s: fido_rx", __func__);
 			r = FIDO_ERR_RX;
 			goto fail;
 		}
-		if (usleep((ms == -1 ? 100 : ms) * 1000) < 0) {
-			fido_log_debug("%s: usleep", __func__);
+		if (delay_ms(U2F_PACE_MS, ms) != 0) {
+			fido_log_debug("%s: delay_ms", __func__);
 			r = FIDO_ERR_RX;
 			goto fail;
 		}
@@ -400,6 +411,71 @@ fail:
 	return (ok);
 }
 
+static int
+encode_cred_attstmt(int cose_alg, const fido_blob_t *x5c,
+    const fido_blob_t *sig, fido_blob_t *out)
+{
+	cbor_item_t		*item = NULL;
+	cbor_item_t		*x5c_cbor = NULL;
+	const uint8_t		 alg_cbor = (uint8_t)(-cose_alg - 1);
+	struct cbor_pair	 kv[3];
+	size_t			 alloc_len;
+	int			 ok = -1;
+
+	memset(&kv, 0, sizeof(kv));
+	memset(out, 0, sizeof(*out));
+
+	if ((item = cbor_new_definite_map(3)) == NULL) {
+		fido_log_debug("%s: cbor_new_definite_map", __func__);
+		goto fail;
+	}
+
+	if ((kv[0].key = cbor_build_string("alg")) == NULL ||
+	    (kv[0].value = cbor_build_negint8(alg_cbor)) == NULL ||
+	    !cbor_map_add(item, kv[0])) {
+		fido_log_debug("%s: alg", __func__);
+		goto fail;
+	}
+
+	if ((kv[1].key = cbor_build_string("sig")) == NULL ||
+	    (kv[1].value = fido_blob_encode(sig)) == NULL ||
+	    !cbor_map_add(item, kv[1])) {
+		fido_log_debug("%s: sig", __func__);
+		goto fail;
+	}
+
+	if ((kv[2].key = cbor_build_string("x5c")) == NULL ||
+	    (kv[2].value = cbor_new_definite_array(1)) == NULL ||
+	    (x5c_cbor = fido_blob_encode(x5c)) == NULL ||
+	    !cbor_array_push(kv[2].value, x5c_cbor) ||
+	    !cbor_map_add(item, kv[2])) {
+		fido_log_debug("%s: x5c", __func__);
+		goto fail;
+	}
+
+	if ((out->len = cbor_serialize_alloc(item, &out->ptr,
+	    &alloc_len)) == 0) {
+		fido_log_debug("%s: cbor_serialize_alloc", __func__);
+		goto fail;
+	}
+
+	ok = 0;
+fail:
+	if (item != NULL)
+		cbor_decref(&item);
+	if (x5c_cbor != NULL)
+		cbor_decref(&x5c_cbor);
+
+	for (size_t i = 0; i < nitems(kv); i++) {
+		if (kv[i].key)
+			cbor_decref(&kv[i].key);
+		if (kv[i].value)
+			cbor_decref(&kv[i].value);
+	}
+
+	return (ok);
+}
+
 static int
 encode_cred_authdata(const char *rp_id, const uint8_t *kh, uint8_t kh_len,
     const uint8_t *pubkey, size_t pubkey_len, fido_blob_t *out)
@@ -474,14 +550,8 @@ fail:
 	if (authdata_cbor)
 		cbor_decref(&authdata_cbor);
 
-	if (pk_blob.ptr) {
-		explicit_bzero(pk_blob.ptr, pk_blob.len);
-		free(pk_blob.ptr);
-	}
-	if (authdata_blob.ptr) {
-		explicit_bzero(authdata_blob.ptr, authdata_blob.len);
-		free(authdata_blob.ptr);
-	}
+	fido_blob_reset(&pk_blob);
+	fido_blob_reset(&authdata_blob);
 
 	return (ok);
 }
@@ -492,6 +562,7 @@ parse_register_reply(fido_cred_t *cred, const unsigned char *reply, size_t len)
 	fido_blob_t	 x5c;
 	fido_blob_t	 sig;
 	fido_blob_t	 ad;
+	fido_blob_t	 stmt;
 	uint8_t		 dummy;
 	uint8_t		 pubkey[65];
 	uint8_t		 kh_len = 0;
@@ -501,6 +572,7 @@ parse_register_reply(fido_cred_t *cred, const unsigned char *reply, size_t len)
 	memset(&x5c, 0, sizeof(x5c));
 	memset(&sig, 0, sizeof(sig));
 	memset(&ad, 0, sizeof(ad));
+	memset(&stmt, 0, sizeof(stmt));
 	r = FIDO_ERR_RX;
 
 	/* status word */
@@ -534,6 +606,12 @@ parse_register_reply(fido_cred_t *cred, const unsigned char *reply, size_t len)
 		goto fail;
 	}
 
+	/* attstmt */
+	if (encode_cred_attstmt(COSE_ES256, &x5c, &sig, &stmt) < 0) {
+		fido_log_debug("%s: encode_cred_attstmt", __func__);
+		goto fail;
+	}
+
 	/* authdata */
 	if (encode_cred_authdata(cred->rp.id, kh, kh_len, pubkey,
 	    sizeof(pubkey), &ad) < 0) {
@@ -543,8 +621,7 @@ parse_register_reply(fido_cred_t *cred, const unsigned char *reply, size_t len)
 
 	if (fido_cred_set_fmt(cred, "fido-u2f") != FIDO_OK ||
 	    fido_cred_set_authdata(cred, ad.ptr, ad.len) != FIDO_OK ||
-	    fido_cred_set_x509(cred, x5c.ptr, x5c.len) != FIDO_OK ||
-	    fido_cred_set_sig(cred, sig.ptr, sig.len) != FIDO_OK) {
+	    fido_cred_set_attstmt(cred, stmt.ptr, stmt.len) != FIDO_OK) {
 		fido_log_debug("%s: fido_cred_set", __func__);
 		r = FIDO_ERR_INTERNAL;
 		goto fail;
@@ -552,41 +629,25 @@ parse_register_reply(fido_cred_t *cred, const unsigned char *reply, size_t len)
 
 	r = FIDO_OK;
 fail:
-	if (kh) {
-		explicit_bzero(kh, kh_len);
-		free(kh);
-	}
-	if (x5c.ptr) {
-		explicit_bzero(x5c.ptr, x5c.len);
-		free(x5c.ptr);
-	}
-	if (sig.ptr) {
-		explicit_bzero(sig.ptr, sig.len);
-		free(sig.ptr);
-	}
-	if (ad.ptr) {
-		explicit_bzero(ad.ptr, ad.len);
-		free(ad.ptr);
-	}
+	freezero(kh, kh_len);
+	fido_blob_reset(&x5c);
+	fido_blob_reset(&sig);
+	fido_blob_reset(&ad);
+	fido_blob_reset(&stmt);
 
 	return (r);
 }
 
 int
-u2f_register(fido_dev_t *dev, fido_cred_t *cred, int ms)
+u2f_register(fido_dev_t *dev, fido_cred_t *cred, int *ms)
 {
-	const uint8_t	 cmd = CTAP_FRAME_INIT | CTAP_CMD_MSG;
 	iso7816_apdu_t	*apdu = NULL;
 	unsigned char	 rp_id_hash[SHA256_DIGEST_LENGTH];
-	unsigned char	 reply[2048];
+	unsigned char	 reply[FIDO_MAXMSG];
 	int		 reply_len;
 	int		 found;
 	int		 r;
 
-#ifdef FIDO_FUZZ
-	ms = 0; /* XXX */
-#endif
-
 	if (cred->rk == FIDO_OPT_TRUE || cred->uv == FIDO_OPT_TRUE) {
 		fido_log_debug("%s: rk=%d, uv=%d", __func__, cred->rk,
 		    cred->uv);
@@ -624,7 +685,7 @@ u2f_register(fido_dev_t *dev, fido_cred_t *cred, int ms)
 		return (FIDO_ERR_INTERNAL);
 	}
 
-	if ((apdu = iso7816_new(U2F_CMD_REGISTER, 0, 2 *
+	if ((apdu = iso7816_new(0, U2F_CMD_REGISTER, 0, 2 *
 	    SHA256_DIGEST_LENGTH)) == NULL ||
 	    iso7816_add(apdu, cred->cdh.ptr, cred->cdh.len) < 0 ||
 	    iso7816_add(apdu, rp_id_hash, sizeof(rp_id_hash)) < 0) {
@@ -634,20 +695,20 @@ u2f_register(fido_dev_t *dev, fido_cred_t *cred, int ms)
 	}
 
 	do {
-		if (fido_tx(dev, cmd, iso7816_ptr(apdu),
-		    iso7816_len(apdu)) < 0) {
+		if (fido_tx(dev, CTAP_CMD_MSG, iso7816_ptr(apdu),
+		    iso7816_len(apdu), ms) < 0) {
 			fido_log_debug("%s: fido_tx", __func__);
 			r = FIDO_ERR_TX;
 			goto fail;
 		}
-		if ((reply_len = fido_rx(dev, cmd, &reply, sizeof(reply),
-		    ms)) < 2) {
+		if ((reply_len = fido_rx(dev, CTAP_CMD_MSG, &reply,
+		    sizeof(reply), ms)) < 2) {
 			fido_log_debug("%s: fido_rx", __func__);
 			r = FIDO_ERR_RX;
 			goto fail;
 		}
-		if (usleep((ms == -1 ? 100 : ms) * 1000) < 0) {
-			fido_log_debug("%s: usleep", __func__);
+		if (delay_ms(U2F_PACE_MS, ms) != 0) {
+			fido_log_debug("%s: delay_ms", __func__);
 			r = FIDO_ERR_RX;
 			goto fail;
 		}
@@ -666,7 +727,7 @@ fail:
 
 static int
 u2f_authenticate_single(fido_dev_t *dev, const fido_blob_t *key_id,
-    fido_assert_t *fa, size_t idx, int ms)
+    fido_assert_t *fa, size_t idx, int *ms)
 {
 	fido_blob_t	sig;
 	fido_blob_t	ad;
@@ -687,6 +748,12 @@ u2f_authenticate_single(fido_dev_t *dev, const fido_blob_t *key_id,
 		goto fail;
 	}
 
+	if (fido_blob_set(&fa->stmt[idx].id, key_id->ptr, key_id->len) < 0) {
+		fido_log_debug("%s: fido_blob_set", __func__);
+		r = FIDO_ERR_INTERNAL;
+		goto fail;
+	}
+
 	if (fa->up == FIDO_OPT_FALSE) {
 		fido_log_debug("%s: checking for key existence only", __func__);
 		r = FIDO_ERR_USER_PRESENCE_REQUIRED;
@@ -699,8 +766,7 @@ u2f_authenticate_single(fido_dev_t *dev, const fido_blob_t *key_id,
 		goto fail;
 	}
 
-	if (fido_blob_set(&fa->stmt[idx].id, key_id->ptr, key_id->len) < 0 ||
-	    fido_assert_set_authdata(fa, idx, ad.ptr, ad.len) != FIDO_OK ||
+	if (fido_assert_set_authdata(fa, idx, ad.ptr, ad.len) != FIDO_OK ||
 	    fido_assert_set_sig(fa, idx, sig.ptr, sig.len) != FIDO_OK) {
 		fido_log_debug("%s: fido_assert_set", __func__);
 		r = FIDO_ERR_INTERNAL;
@@ -709,22 +775,17 @@ u2f_authenticate_single(fido_dev_t *dev, const fido_blob_t *key_id,
 
 	r = FIDO_OK;
 fail:
-	if (sig.ptr) {
-		explicit_bzero(sig.ptr, sig.len);
-		free(sig.ptr);
-	}
-	if (ad.ptr) {
-		explicit_bzero(ad.ptr, ad.len);
-		free(ad.ptr);
-	}
+	fido_blob_reset(&sig);
+	fido_blob_reset(&ad);
 
 	return (r);
 }
 
 int
-u2f_authenticate(fido_dev_t *dev, fido_assert_t *fa, int ms)
+u2f_authenticate(fido_dev_t *dev, fido_assert_t *fa, int *ms)
 {
-	int	nauth_ok = 0;
+	size_t	nfound = 0;
+	size_t	nauth_ok = 0;
 	int	r;
 
 	if (fa->uv == FIDO_OPT_TRUE || fa->allow_list.ptr == NULL) {
@@ -739,20 +800,111 @@ u2f_authenticate(fido_dev_t *dev, fido_assert_t *fa, int ms)
 	}
 
 	for (size_t i = 0; i < fa->allow_list.len; i++) {
-		if ((r = u2f_authenticate_single(dev, &fa->allow_list.ptr[i],
-		    fa, nauth_ok, ms)) == FIDO_OK) {
+		switch ((r = u2f_authenticate_single(dev,
+		    &fa->allow_list.ptr[i], fa, nfound, ms))) {
+		case FIDO_OK:
 			nauth_ok++;
-		} else if (r != FIDO_ERR_CREDENTIAL_EXCLUDED) {
-			fido_log_debug("%s: u2f_authenticate_single", __func__);
-			return (r);
+			/* FALLTHROUGH */
+		case FIDO_ERR_USER_PRESENCE_REQUIRED:
+			nfound++;
+			break;
+		default:
+			if (r != FIDO_ERR_CREDENTIAL_EXCLUDED) {
+				fido_log_debug("%s: u2f_authenticate_single",
+				    __func__);
+				return (r);
+			}
+			/* ignore credentials that don't exist */
 		}
-		/* ignore credentials that don't exist */
 	}
 
-	fa->stmt_len = nauth_ok;
+	fa->stmt_len = nfound;
 
-	if (nauth_ok == 0)
+	if (nfound == 0)
 		return (FIDO_ERR_NO_CREDENTIALS);
+	if (nauth_ok == 0)
+		return (FIDO_ERR_USER_PRESENCE_REQUIRED);
+
+	return (FIDO_OK);
+}
+
+int
+u2f_get_touch_begin(fido_dev_t *dev, int *ms)
+{
+	iso7816_apdu_t	*apdu = NULL;
+	const char	*clientdata = FIDO_DUMMY_CLIENTDATA;
+	const char	*rp_id = FIDO_DUMMY_RP_ID;
+	unsigned char	 clientdata_hash[SHA256_DIGEST_LENGTH];
+	unsigned char	 rp_id_hash[SHA256_DIGEST_LENGTH];
+	unsigned char	 reply[FIDO_MAXMSG];
+	int		 r;
+
+	memset(&clientdata_hash, 0, sizeof(clientdata_hash));
+	memset(&rp_id_hash, 0, sizeof(rp_id_hash));
+
+	if (SHA256((const void *)clientdata, strlen(clientdata),
+	    clientdata_hash) != clientdata_hash || SHA256((const void *)rp_id,
+	    strlen(rp_id), rp_id_hash) != rp_id_hash) {
+		fido_log_debug("%s: sha256", __func__);
+		return (FIDO_ERR_INTERNAL);
+	}
+
+	if ((apdu = iso7816_new(0, U2F_CMD_REGISTER, 0, 2 *
+	    SHA256_DIGEST_LENGTH)) == NULL ||
+	    iso7816_add(apdu, clientdata_hash, sizeof(clientdata_hash)) < 0 ||
+	    iso7816_add(apdu, rp_id_hash, sizeof(rp_id_hash)) < 0) {
+		fido_log_debug("%s: iso7816", __func__);
+		r = FIDO_ERR_INTERNAL;
+		goto fail;
+	}
+
+	if (dev->attr.flags & FIDO_CAP_WINK) {
+		fido_tx(dev, CTAP_CMD_WINK, NULL, 0, ms);
+		fido_rx(dev, CTAP_CMD_WINK, &reply, sizeof(reply), ms);
+	}
+
+	if (fido_tx(dev, CTAP_CMD_MSG, iso7816_ptr(apdu),
+	    iso7816_len(apdu), ms) < 0) {
+		fido_log_debug("%s: fido_tx", __func__);
+		r = FIDO_ERR_TX;
+		goto fail;
+	}
+
+	r = FIDO_OK;
+fail:
+	iso7816_free(&apdu);
+
+	return (r);
+}
+
+int
+u2f_get_touch_status(fido_dev_t *dev, int *touched, int *ms)
+{
+	unsigned char	reply[FIDO_MAXMSG];
+	int		reply_len;
+	int		r;
+
+	if ((reply_len = fido_rx(dev, CTAP_CMD_MSG, &reply, sizeof(reply),
+	    ms)) < 2) {
+		fido_log_debug("%s: fido_rx", __func__);
+		return (FIDO_OK); /* ignore */
+	}
+
+	switch ((reply[reply_len - 2] << 8) | reply[reply_len - 1]) {
+	case SW_CONDITIONS_NOT_SATISFIED:
+		if ((r = u2f_get_touch_begin(dev, ms)) != FIDO_OK) {
+			fido_log_debug("%s: u2f_get_touch_begin", __func__);
+			return (r);
+		}
+		*touched = 0;
+		break;
+	case SW_NO_ERROR:
+		*touched = 1;
+		break;
+	default:
+		fido_log_debug("%s: unexpected sw", __func__);
+		return (FIDO_ERR_RX);
+	}
 
 	return (FIDO_OK);
 }
